#ifndef FITS_H
#define FITS_H

/* $Header: /rosie/src/ccd_utilities/include/RCS/fits.h,v 1.9 1996/05/07 17:21:23 tjk Exp $
 */

#define		MAX_DIRECTORY_SIZE	127
#define		MAX_FILENAME_SIZE	31
#define		MAX_BUF_SIZE		255
#define		MAX_PATH_NAME		158
#define		FITS_HEADER_SIZE	2880
#define		DEFAULT_FITS_FILENAME		"TEST"
#define		MAX_IMAGE_SIZE		90

/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
#define		DEFAULT_FITS_AIRMASS		99.9
#define		DEFAULT_FITS_BITPIX		16
#define		DEFAULT_FITS_DATE_OBS		"date"
#define		DEFAULT_FITS_DEC		"dec"
#define		DEFAULT_FITS_DETECTOR		"detector"
#define		DEFAULT_FITS_EPOCH		1996.5
#define		DEFAULT_FITS_EXPTIME		1.000
#define		DEFAULT_FITS_EXPTYPE		"exposure type"
#define		DEFAULT_FITS_FILTER		"filter"
#define		DEFAULT_FITS_INSTRUMENT		"instrument"
#define		DEFAULT_FITS_IRAFNAME		"TEST"
#define		DEFAULT_FITS_ISTART		10
#define		DEFAULT_FITS_JSTART		10
#define		DEFAULT_FITS_NAXIS		2
#define		DEFAULT_FITS_NAXIS1		200
#define		DEFAULT_FITS_NAXIS2		1000
#define		DEFAULT_FITS_OBJECT		"object"
#define		DEFAULT_FITS_OBSERVATORY	"Mount Laguna Observatory"
#define		DEFAULT_FITS_OBSERVER		"observer"
#define		DEFAULT_FITS_POSTSCAN		0
#define		DEFAULT_FITS_PRESCAN		0
#define		DEFAULT_FITS_RA			"right ascension"
#define		DEFAULT_FITS_SIDEREAL_TIME	"sidereal time"
#define		DEFAULT_FITS_SIMPLE		'T'
#define		DEFAULT_FITS_TELESCOPE		"MLO 40-inch Telescope"
#define		DEFAULT_FITS_UTSTART		"ut"

#define		DEFAULT_FITS_KEYWORD		""

#define		DEFAULT_FITS_KEYWORD_VALUE	""

#define		FITS_FD			1
#define		FITS_IMAGE_ADDR		2
#define		FITS_DIRECTORY		3
#define		FITS_FILENAME		4

/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */

/* FITS_BEGIN_TAGS should correspond to the first tag in the list.     */
#define		FITS_BEGIN_TAGS		5

#define		FITS_AIRMASS		5
#define		FITS_BITPIX		6
#define		FITS_BLANK		7
#define		FITS_COMMENT		8
#define		FITS_DATE_OBS		9
#define		FITS_DEC		10
#define		FITS_DETECTOR		11
#define		FITS_END		12
#define		FITS_EPOCH		13
#define		FITS_EXPTIME		14
#define		FITS_EXPTYPE		15
#define		FITS_FILTER		16
#define		FITS_HISTORY		17
#define		FITS_INSTRUMENT		18
#define		FITS_IRAFNAME		19
#define		FITS_ISTART		20
#define		FITS_JSTART		21
#define		FITS_NAXIS		22
#define		FITS_NAXIS1		23
#define		FITS_NAXIS2		24
#define		FITS_OBJECT		25
#define		FITS_OBSERVATORY	26
#define		FITS_OBSERVER		27
#define		FITS_POSTSCAN		28
#define		FITS_PRESCAN		29
#define		FITS_RA			30
#define		FITS_SIDEREAL_TIME	31
#define		FITS_SIMPLE		32
#define		FITS_TELESCOPE		33
#define		FITS_UTSTART		34

/* This tag should be the last numbered tag in the FITS_ tags, although *\
 * it is not a true tag itself.  This tag is used for the free form     *
\* comments section.                                                    */
#define		FITS_USER_COMMENTS	35


/* FITS_END_TAGS should correspond to the last tag in the list.     */
#define		FITS_END_TAGS		35

#define		MAX_FITS_TAGS		(FITS_END_TAGS - FITS_BEGIN_TAGS)
#define		MAX_USER_COMMENTS	36

/* These FITS tags are used for user input.
 * The user is allowed to enter up to ten (10) customized fields.
 */
#define		FITS_USER_KEYWORD1	101
#define		FITS_USER_KEYWORD2	102
#define		FITS_USER_KEYWORD3	103
#define		FITS_USER_KEYWORD4	104
#define		FITS_USER_KEYWORD5	105
#define		FITS_USER_KEYWORD6	106
#define		FITS_USER_KEYWORD7	107
#define		FITS_USER_KEYWORD8	108
#define		FITS_USER_KEYWORD9	109

#define		FITS_USER_KEYWORDS	110
#define		MAX_USER_KEYWORDS     (FITS_USER_KEYWORDS - FITS_USER_KEYWORD1)

#define		FITS_USER_KEYWORD_VALUE1	201
#define		FITS_USER_KEYWORD_VALUE2	202
#define		FITS_USER_KEYWORD_VALUE3	203
#define		FITS_USER_KEYWORD_VALUE4	204
#define		FITS_USER_KEYWORD_VALUE5	205
#define		FITS_USER_KEYWORD_VALUE6	206
#define		FITS_USER_KEYWORD_VALUE7	207
#define		FITS_USER_KEYWORD_VALUE8	208
#define		FITS_USER_KEYWORD_VALUE9	209


#define		FITS_OPEN_ERR		1
#define		FITS_CLOSE_ERR		2

#define	FALSE	0
#define TRUE	1

/* BITPIX values */
#define	BITPIX_SETTING_0	0
#define	BITPIX_SETTING_1	1
#define	BITPIX_SETTING_2	2
#define	BITPIX_SETTING_3	3
#define	BITPIX_SETTING_4	4

#define BITPIX_VALUE_0		  8
#define BITPIX_VALUE_1		 16
#define BITPIX_VALUE_2		 32
#define BITPIX_VALUE_3		-32
#define BITPIX_VALUE_4		-64

/*>>>>> User-defined types <<<<<*/
typedef unsigned long   fits_opaque;

typedef struct _fios {
        int     tag;
        char    string[10];
} FIOS;
 
typedef struct _fits_obj {
	int		fd;
	caddr_t		image_addr;
	char		directory[MAX_DIRECTORY_SIZE];
	char		filename[MAX_FILENAME_SIZE];

	/* FITS Tags */
/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
	float		airmass;
	int		bitpix;
	char		comments[MAX_USER_COMMENTS][71];
	char		date_observation[71];
	char		dec[71];
	char		detector[71];
	float		epoch;
	float		exptime;
	char		exptype[71];
	char		filter[71];
	char		instrument[71];
	char		iraf_name[MAX_FILENAME_SIZE];
	int		istart;
	int		jstart;
	int		naxis;
	int		naxis1;
	int		naxis2;
	char		object[71];
	char		observatory[71];
	char		observer[71];
	int		postscan;
	int		prescan;
	char		ra[71];
	char		sidereal_time[71];
	char		simple;
	char		telescope[71];
	char		ut_start[71];
	char		user_keywords[MAX_USER_KEYWORDS][9];
	char		user_keyword_values[MAX_USER_KEYWORDS][71];
} FITS_OBJ;

typedef int	FITS_TAGS;


/*>>>>> Function Prototypes <<<<<*/
#ifdef __STDC__
static	void		clear_image_mem(FITS_OBJ *obj, int size);
extern	int		fits_close(FITS_OBJ *obj);
extern	fits_opaque	*fits_create(void);
extern	void		fits_create_card(...);
extern	char		*fits_format(FITS_OBJ *fits, int fits_item);
extern	fits_opaque	fits_get(...);
extern	int		fits_open(FITS_OBJ *obj);
extern	void		fits_perror(char *s);
extern	int		fits_set(...);
extern	int		fits_write(FITS_OBJ *obj);
extern	void		put_fits_header(FITS_OBJ *obj);
extern	void		write_card(int fd, char *str);

#else	/* K&R C */
static	void		clear_image_mem();
extern	int		fits_close();
extern	fits_opaque	*fits_create();
extern	void		fits_create_card();
extern	char		*fits_format();
extern	fits_opaque	fits_get();
extern	int		fits_open();
extern	void		fits_perror();
extern	int		fits_set();
extern	int		fits_write();
extern	void		put_fits_header();
extern	void		write_card();
#endif /* __STDC__ */

/*>>>>> External Variables <<<<<*/
extern	int	fits_errno;
extern	char	*fits_errlist[];
extern	FIOS	fios[];

/*
 * $Log: fits.h,v $
 * Revision 1.9  1996/05/07 17:21:23  tjk
 * Added user supplied keywords
 *
 * Revision 1.8  1996/02/14 18:20:21  tjk
 * Changed some default FITS values.
 *
 * Revision 1.7  1995/12/12  22:33:18  tjk
 * Merged SBUS and VME versions
 *
 * Revision 1.6  1995/08/11  03:28:22  tjk
 * converting to ANSI C standards
 *
 * Revision 1.5  1995/08/11  03:14:11  tjk
 * converting to ANSI C standards
 *
 * Revision 1.4  1995/08/10  16:14:12  tjk
 * Added more FITS stuff
 *
 * Revision 1.3  1995/06/13  23:53:02  tjk
 * Added more fits stuff.
 *
 * Revision 1.2  1995/02/22  20:39:05  tjk
 * Added more FITS header information.
 *
 * Revision 1.1  1995/02/10  03:22:01  tjk
 * Initial revision
 *
 *
 */

#endif FITS_H
