#ifndef _sao_iis_h
#define _sao_iis_h

/*
 *  sao-iis.h
 *
 *  header file for the sao-iis library.
 *
 */


#define TRUE            1
#define FALSE           0

#define PASS            0
#define FAIL            -1

#define NAMELEN         128

#define INNAME		"INDEV"		/* env-var for named pipe */
#define OUTNAME		"OUTDEV"	/* env-var for named pipe */

#define	I_DEVNAME	"imt1i"		/* default name of named pipe */
#define	O_DEVNAME	"imt1o"		/* default name of named pipe */

#define	MEMORY		01		/* frame buffer i/o		 */
#define	LUT		02		/* lut i/o			 */
#define	FEEDBACK	05		/* used for frame clears	 */
#define	IMCURSOR	020		/* logical image cursor		 */
#define	WCS		021		/* used to set WCS		 */

#define	SZ_IMCURVAL	160
#define	PACKED		0040000
#define	COMMAND		0100000
#define	IIS_READ	0100000		/* bit 15 set -> read operation */
#define	IIS_WRITE	0000000		/* bit 15 clear -> write operation */
#define	IMC_SAMPLE	0040000
#define	IMT_FBCONFIG	077

#define MAGIC_BEAN	0100000
#define PACKET_SIZE     32768

#define MIN_NPIXELS     5            /* smallest permissible sample          */
#define MAX_REJECT      0.5          /* max frac. of pixels to be rejected   */
#define GOOD_PIXEL      0            /* use pixel in fit                     */
#define BAD_PIXEL       1            /* ignore pixel in all computations     */
#define REJECT_PIXEL    2            /* reject pixel after a bit             */
#define KREJ            2.5          /* k-sigma pixel rejection factor       */
#define MAX_ITERATIONS  5            /* maximum number of fitline iterations */
#define INDEF           0
 
#undef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#undef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#undef mod
#define mod(a,b) ((a) % (b))
#undef nint
#define nint(a) ((int)(a + 0.5))


/* 
 * this is what your application thinks pixels are, can be real or integer
 */

typedef unsigned short Rawpix_t;

/*
 * this is what saoimage expects, don't change this 
 */

typedef unsigned char IISpix_t;


typedef struct {
    short tid;
    short thingct;
    short subunit;
    short checksum;
    short x, y, z;
    short t;
} IISM70;

extern int si_datain;
extern int si_dataout;

/*******************************
 *     Functions Prototypes    *
********************************/

#ifdef __STDC__

static void si_checksum(IISM70 * iis);
static int si_write(char *buf, int size);
static int si_flush(void);
extern int si_open(void);
extern void si_close(void);
extern int si_set_fb_index(int n, int m, int *config_number,
                           int *width, int *height);
extern int si_send_image(int w, int h, Rawpix_t *im);
extern int si_open_namedpipe(void);
extern void si_close_namedpipe(void);
extern int si_flush_namedpipe(void);
extern int si_send_erase(void);
extern int si_send_wcs(int w, int h, float z1, float z2);


/*******************************/

#else   /* K&R  C */
 
static void si_checksum();
static int si_write();
static int si_flush();
extern int si_open();
extern void si_close();
extern int si_send_image();
extern int si_open_namedpipe();
extern void si_close_namedpipe();
extern int si_flush_namedpipe();
extern int si_send_erase();
extern int si_set_fb_index();
extern int si_send_wcs();


#endif /* __STDC__ */

/*
 * function prototypes for zscale.c
 */

#ifdef __STDC__

extern void zscale (char *im, int nx, int ny, int bitpix, float *z1, float *z2,
                    float contrast, int opt_size, int len_stdline);
int sampleImage (char *im, int bitpix, float **sample, int nx, int ny,
                 int optimal_size, int len_stdline);
static void subSample(float *a, float *b, int npix, int step);
int fitLine(float *data, int npix, float *zstart, float *zslope, float krej,
             int ngrow, int maxiter);
static void flattenData(float *data, float *flat, float *x, int npix,
                        float z0, float dz);
int computeSigma (float *a, short *badpix, int npix, float *mean,
                  float * sigma);
int rejectPixels(float *data, float *flat, float * normx, short *badpix,
                 int npix, double *sumxsqr, double *sumxz, double *sumx,
                 double *sumz, float threshold, int ngrow);
int floatCompare(const void *i, const void *j);
extern void doZscale(unsigned char **pix, int nx, int ny, int bitpix, 
               float z1, float z2, int zt);

#else /* K & R */

extern void zscale();
int sampleImage();
static void subSample();
int fitLine();
static void flattenData();
int computeSigma();
int rejectPixels();
int floatCompare();
extern void  doZscale();

#endif /* __STDC__ */

#endif /* _sao_iis_h */
