#ifndef TEMPERATURE_H
#define TEMPERATURE_H

/* $Header: /rosie/src/ccd_utilities/include/RCS/temperature.h,v 1.3 1995/08/21 19:05:36 moore Exp $ */

/*************************************************************************\
 *  This function calculate the temperature of the the ccd using the     *
 *  Chebychev series and the coefficients given for the CY7 Series       *
 *  Silicon Diodes Standard Curve #10 manufactured by Omega Engineering. *
 *  For this diode, there are eleven (11) coefficients.  Also, the adu-  *
 *  to-voltage conversion factor is needed to use the formula given by   *
 *  Omega Engineering.                                                   *
 *                                                                       *
 *  The function ccd_temperature takes an array of these coefficients,   *
 *  the number of these coefficients, upper voltage limit, lower voltage *
 *  limit, and the adu count as parameters.  The function is declared in *
 *  both ANSI C and K&R C for compatability purposes.  __STDC__ is the   *
 *  variable which signifies ANSI C.                                     *
\*************************************************************************/

#define N_MAX	11

#define DEFAULT_NTEMP		N_MAX

#define DEFAULT_TEMP_COEFF_0	 287.756797
#define DEFAULT_TEMP_COEFF_1	-194.144823
#define DEFAULT_TEMP_COEFF_2	  -3.837903
#define DEFAULT_TEMP_COEFF_3	  -1.318325
#define DEFAULT_TEMP_COEFF_4	  -0.109120
#define DEFAULT_TEMP_COEFF_5	  -0.393265
#define DEFAULT_TEMP_COEFF_6	   0.146911
#define DEFAULT_TEMP_COEFF_7	  -0.111192
#define DEFAULT_TEMP_COEFF_8	   0.028877
#define DEFAULT_TEMP_COEFF_9	  -0.029286
#define DEFAULT_TEMP_COEFF_10	   0.015619

#define DEFAULT_VU		0.999614
#define DEFAULT_VL		0.079767

#define DEFAULT_ADU_PER_VOLT	1366.98
#define DEFAULT_ADU_OFFSET	2045

#ifdef	IRTOOL
#define	MAX_TEMPERATURE_CHECKS	3
#else
#define	MAX_TEMPERATURE_CHECKS	10
#endif	/* IRTOOL */

/*>>>>>>>>>>  Function Prototypes <<<<<<<<<<*/
#ifdef	__STDC__
float ccd_temperature(float temp_coeff[], int n,
		       float vu, float vl,
		       float adu_per_volt,
		       int adu_offset,
		       float adu);
int calc_temp_adu(float temp_coeff[], int n,
		       float vu, float vl,
		       float adu_per_volt,
		       int adu_offset,
		       float temperature);
#else	/* K&R C */
float ccd_temperature();
int calc_temp_adu();
#endif	/* __STDC__ */

/*
 * $Log: temperature.h,v $
 * Revision 1.3  1995/08/21 19:05:36  moore
 * ANSI conversion
 *
 * Revision 1.2  1995/08/10 16:14:31  tjk
 * fixed bugs...
 *
 * Revision 1.1  1995/05/23  21:56:15  tjk
 * Initial revision
 * 
 *
 */

#endif TEMPERATURE_H
