 /* $Header: /rosie/src/ccd_utilities/src/libdsp/RCS/libdsp.c,v 1.81 1997/01/03 22:55:30 gumrukcu Exp $
 */

#include        <stdio.h>
#include        <fcntl.h>
#include        <sys/mman.h>
#include        <stdarg.h> 
#include 	<xview/xview.h>
#include 	"ccd_conf.h"
#include 	"ccd.h"

#include	<signal.h>
#include	<errno.h>


#include	"dsp_conf.h"
#include	"dsp.h"
#include	"fits.h"

#define		TRUE		1
#define		FALSE		0
#define		OK		1
#define		NOT_OK		0
#define		DSP_END		-123456

char	*dsp_errlist[] =
{
	"no error",
	"cannot open SBUS Interface Board",
	"error closing SBUS Interface Board",
	"bad DSP reply header",
	"bad DSP reply",
	"DSP reply timout",
	"cannot allocate memory for DSP attributes",
	"cannot open download file",
	"board type mismatch in download file",
	"download file error",
	"bad attribute in dsp_set"
};

union	reply_buffer {
	char	ch[256];
	u_int	uin[64];
};

union	exp_data {
	char	ch[MAX_EXPOSURE_BYTES];
	u_int	uin[MAX_EXPOSURE_BYTES/4];
	u_short	ush[MAX_EXPOSURE_BYTES/2];
};

union	exp_data	expdata;

int     extr;

static	char	*exp_byte;
int dspmsecs;

		/*>>>>>  Function Prototypes  <<<<<*/

dsp_opaque	*dsp_create(void);
int		dsp_set(...);
dsp_opaque	dsp_get(...);
int		dsp_open(DSP_ATTR *attr);
int		dsp_close(DSP_ATTR *attr);
int		dsp_command(DSP_ATTR *attr,...);
int		send_command(...);
void		send_boot(DSP_ATTR *attr, int data);
void		sigbus_boot(int);
int		send_wrm(DSP_ATTR *attr, int board_id, char mem_space,
		         int address, int data);
/*int		send_set(DSP_ATTR *attr, imt board_id, int data);*/
int		send_rdm(DSP_ATTR *attr, int board_id, char mem_space,
			 int address);
int		send_src(DSP_ATTR *attr, int board_id, int address);
int		send_lda(DSP_ATTR *attr, int board_id, int data);
int		send_sex(DSP_ATTR *attr, int board_id);
int 		send_rrr(DSP_ATTR *attr, int board_id);
int		send_pon(DSP_ATTR *attr, int board_id);
int		send_con(DSP_ATTR *attr, int board_id);
int		send_hgn(DSP_ATTR *attr, int board_id);
int		send_lgn(DSP_ATTR *attr, int board_id);
int		send_rad(DSP_ATTR *attr, int board_id);
int		send_idl(DSP_ATTR *attr, int board_id);
int		send_stp(DSP_ATTR *attr, int board_id);
int		send_clr(DSP_ATTR *attr, int board_id);
int		send_osh(DSP_ATTR *attr, int board_id);
int		send_csh(DSP_ATTR *attr, int board_id);
int		send_rst(DSP_ATTR *attr, int board_id);
int		send_abr(DSP_ATTR *attr, int board_id);
int		send_rdc(DSP_ATTR *attr, int board_id);
int		send_sbv(DSP_ATTR *attr, int board_id);
int		send_tck(DSP_ATTR *attr, int board_id);
int		send_tdc(DSP_ATTR *attr, int board_id);
int		send_aex(DSP_ATTR *attr, int board_id);
int		send_pex(DSP_ATTR *attr, int board_id);
int		send_rex(DSP_ATTR *attr, int board_id);
int		send_syr(DSP_ATTR *attr, int board_id);
int		send_tad(DSP_ATTR *attr, int board_id);
int		send_tda(DSP_ATTR *attr, int board_id);
int		send_tdg(DSP_ATTR *attr, int board_id);
int		send_tdl(DSP_ATTR *attr, int board_id, int data);
int		send_mm1(DSP_ATTR *attr, int board_id, int data);
int		send_mh1(DSP_ATTR *attr, int board_id);
int		send_mm2(DSP_ATTR *attr, int board_id, int data);
int		send_mh2(DSP_ATTR *attr, int board_id);
int		dsp_get_reply(DSP_ATTR *attr, int board_id,
			      int expected_reply, int timeout);
int		dsp_poll(DSP_ATTR *attr, int board_id, int expected_reply);
void		dsp_update_offset(DSP_ATTR *attr);
int		dsp_download(DSP_ATTR *attr, int board, char *filename);
static int	dsp_read_line(FILE *fp, char *buf);
static int	dsp_get_download_type(FILE *fp);
static int	dsp_process_data(DSP_ATTR *attr, FILE *download_fp,
				 int board_id, char type, int addr);
void		dsp_perror(char *s);
void		send_reset(DSP_ATTR *attr);
static void	clear_image_buf(int size);
static void	report_errno(DSP_ATTR *attr,int local_errno);
static void	clear_reply_mem(int size);
void		disp_reply_mem();
extern int	save_image(char *directory,char *filename,int numbytes);

		/*>>>>>  End Function Prototypes  <<<<<*/

/*extern		int	errno;*/
int		dsp_errno;
int		dsp_bad_header_count;

#ifdef	SBUSTOOL
union	reply_buffer dat;
int	*reply_arg = (int *)&dat.uin[0];
char	*reply_byte = (char *)&dat.ch[0];
#endif	/* SBUSTOOL */

extern int pause_flag;
extern int abort_flag;
extern int time_flag;
extern int beeper;
extern int dual_image_mode;


#ifdef  __STDC__
dsp_opaque *dsp_create(void)
#else /* K&R C */
dsp_opaque
*dsp_create()
#endif	/* __STDC__ */
{
	
	DSP_ATTR	*attr;

	if ((attr = (DSP_ATTR *)malloc(sizeof(DSP_ATTR))) != NULL) {
		attr->sbus_dev0[0] = NULL;
		attr->sbus_dev1[0] = NULL;
		attr->sbus_request = DEFAULT_SBUS_REQUEST;
		attr->reply_action = DEFAULT_REPLY_ACTION;
		attr->reply_func_ptr = NULL;
		attr->print_ptr = NULL;
		attr->abort = FALSE;
		attr->debug_level = DEFAULT_DSP_DEBUG_LEVEL;
		attr->last_header = 0;
		attr->last_reply = 0;
		attr->first_done = FALSE;

		return (dsp_opaque *)attr;
	}
	else {
		dsp_errno = DSP_MALLOC_ERROR;
		message_write("BURADANm GELDI Faik\n");
		return (dsp_opaque *)NULL;
	}

}

int dsp_set(...)
{
	va_list 	ap;
	DSP_ATTR	*attr;
	int		modifier;
	char		*buffer;

	va_start(ap,DSP_ATTR *);
	attr = va_arg(ap, DSP_ATTR *);
	modifier = va_arg(ap, int);

	while (modifier != NULL) {

	   switch (modifier) {

		case SBUS_DEVICE_ZERO:
			buffer = va_arg(ap, char *);
			strncpy(attr->sbus_dev0,
				buffer,strlen(buffer)+1);
			break;
		case SBUS_DEVICE_ONE:
			buffer = va_arg(ap, char *);
			strncpy(attr->sbus_dev1,
				buffer,strlen(buffer)+1);
			break;
		case SBUS_REQUEST:
			attr->sbus_request = va_arg(ap, int);
			break;
		case REPLY_ACTION:
			attr->reply_action = va_arg(ap, int);
			break;
		case REPLY_FUNC_PTR:
			attr->reply_func_ptr = (int (*)())(va_arg(ap, char *));
			break;
		case DSP_PRINT_PTR:
			attr->print_ptr = (int (*)())(va_arg(ap, char *));
			break;
		case DSP_ABORT:
			attr->abort = va_arg(ap, int);
			break;
		case DSP_DEBUG_LEVEL:
			attr->debug_level = va_arg(ap, int);
			break;
		default:
			dsp_errno = DSP_SET_ERROR;
			va_end(ap);
			return DSP_ERROR;
	   }

	modifier = va_arg(ap, int);
	}
	return DSP_OK;
}

dsp_opaque dsp_get(...)
{
	va_list 	ap;
	DSP_ATTR 	*attr;
	int		modifier;

	va_start(ap, DSP_ATTR *);
	attr = va_arg(ap, DSP_ATTR *);
	modifier = va_arg(ap, int); 

	switch (modifier) {
		case SBUS_DEVICE_ZERO:
			return (dsp_opaque)attr->sbus_dev0;
		case SBUS_DEVICE_ONE:
			return (dsp_opaque)attr->sbus_dev1;
		case SBUS_REQUEST:
			return (dsp_opaque)attr->sbus_request;
		case REPLY_ACTION:
			return (dsp_opaque)attr->reply_action;
		case REPLY_FUNC_PTR:
			return (dsp_opaque)attr->reply_func_ptr;
		case DSP_ABORT:
			return (dsp_opaque)attr->abort;
		case DSP_DEBUG_LEVEL:
			return (dsp_opaque)attr->debug_level;
		case DSP_LAST_HEADER:
			return (dsp_opaque)attr->last_header;
		case DSP_LAST_REPLY:
			return (dsp_opaque)attr->last_reply;
		case DSP_FIRST_DONE:
			return (dsp_opaque)attr->first_done;
	}
}
 

#ifdef __STDC__
int dsp_open(DSP_ATTR *attr)
#else /* K&R C */
int
dsp_open(attr)
DSP_ATTR	*attr;
#endif	/* __STDC__ */
{
#ifdef	SBUSTOOL
	if ((attr->sbus_fd = open(attr->sbus_dev0, O_RDWR)) == -1)
	{
		dsp_errno = SBUS_OPEN_ERROR;
	}

	if (dsp_errno)
	{
		dsp_errno = 0;
		if ((attr->sbus_fd = open(attr->sbus_dev1, O_RDWR)) == -1)
			dsp_errno = SBUS_OPEN_ERROR;
	}

#else	/* VMETOOL */

	if ((attr->mem_fd = open(attr->vme_dev, O_RDWR)) == -1)
	{
		dsp_errno = VME_OPEN_ERROR;
		return DSP_ERROR;
	}

	if ((attr->command_fd = open(attr->vme_dev, O_RDWR)) == -1)
	{
		dsp_errno = VME_OPEN_ERROR;
		return DSP_ERROR;
	}

        if ((attr->mem_ptr = (caddr_t)mmap((caddr_t)0, attr->vme_mem_size,
		PROT_READ | PROT_WRITE, MAP_SHARED, attr->mem_fd,
		(attr->vme_mem_start - attr->vme_mem_offset))) == (caddr_t)-1)
	{
		dsp_errno = VME_MMAP_ERROR;
		return DSP_ERROR;
        }

        if ((attr->command_ptr = (caddr_t)mmap((caddr_t)0, 8,
		PROT_READ | PROT_WRITE, MAP_SHARED, attr->command_fd,
		(attr->vme_send_start - attr->vme_mem_offset))) == (caddr_t)-1)
	{
		dsp_errno = VME_MMAP_ERROR;
		return DSP_ERROR;
        }

	attr->send_addr = (int *)attr->command_ptr;
	/*attr->send_addr = (int *)(attr->mem_ptr + attr->vme_send_start - attr->vme_mem_start); */
	attr->rec_addr = (int *)(attr->mem_ptr + attr->vme_rec_start -
		attr->vme_mem_start);
	attr->data_addr = (int *)(attr->mem_ptr + attr->vme_data_start -
		attr->vme_mem_start);
#endif	/* SBUSTOOL */

	if (dsp_errno)
		return DSP_ERROR;
	else
		return DSP_OK;
}

#ifdef __STDC__
int dsp_close(DSP_ATTR *attr)
#else /* K&R C */
int
dsp_close(attr)
DSP_ATTR	*attr;
#endif	/* __STDC__ */
{

	dsp_errno = 0;

#ifdef	SBUSTOOL
	if ((close(attr->sbus_fd)) == -1)
		dsp_errno = DSP_CLOSE_ERROR;

#else	/* VMETOOL  */
	if (attr->mem_ptr) {
        	if ((munmap(attr->mem_ptr, attr->vme_mem_size)) == -1) {
			dsp_errno = DSP_CLOSE_ERROR;
        	}
	}
        if ((close(attr->mem_fd)) == -1) {
		dsp_errno = DSP_CLOSE_ERROR;
        }
#endif	/* SBUSTOOL */

	if (dsp_errno)
		return DSP_ERROR;
	else 
		return DSP_OK;
}

#ifdef __STDC__
int dsp_command(DSP_ATTR *attr,...)
#else /* K&R C */
int
dsp_command(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list	ap;
	int	board_id, board_header, reply_header;
	int	command, address, data,directory,filename;
	int     ext,numbytes,msecs;
	char	mem_space;

	va_start(ap,NULL);
	board_id = va_arg(ap, int);
	command = va_arg(ap, int);

	if (attr->debug_level != DSP_DO_NOTHING)
	  clear_reply_mem(SBUS_BUFFER_SIZE);

	switch (command) {
		/*  boot commands */
		case LDA:
			data = va_arg(ap, int);
			send_lda(attr, board_id, data);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case RDM:
			mem_space = va_arg(ap, char);
			address = va_arg(ap, int);
			send_rdm(attr, board_id, mem_space, address);
			return(dsp_get_reply(attr, board_id, DSP_ACTUAL_VALUE,
					     DEFAULT_TIMEOUT));
		case RST:
			send_rst(attr, board_id);
			return(dsp_get_reply(attr, board_id, RST,
					     DEFAULT_TIMEOUT));
		case TDL:
			data = va_arg(ap, int);
			send_tdl(attr, board_id, data);
			return(dsp_get_reply(attr, board_id, DSP_ACTUAL_VALUE,
					     DEFAULT_TIMEOUT));
		case WRM:
			mem_space = va_arg(ap, char);
			address = va_arg(ap, int);
			data = va_arg(ap, int);
			send_wrm(attr, board_id, mem_space, address, data);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		/*case SET:
			data = va_arg(ap,int);
			send_set(attr,board_id,data);
			return(dsp_get_reply(attr,board_id, DON,
					     DEFAULT_TIMEOUT));*/

		/* SBUS commands */
		case CSR:
			if(attr->debug_level >= DSP_SHOW_COMMANDS)
        		{
        			if (attr->print_ptr != NULL)
              			attr->print_ptr("\nCommand sent:CSR\n");
         			else
              			fprintf(stdout, "\nCommand sent:CSR\n");
        		}

			attr->sbus_request = SBUS_REQUEST_READ_CSR;
			return(send_dma(attr, attr->sbus_request));
		case ADR:
			
			if(attr->debug_level >= DSP_SHOW_COMMANDS)
        		{
        			if (attr->print_ptr != NULL)
              			attr->print_ptr("\nCommand sent:ADR\n");
         			else
              			fprintf(stdout, "\nCommand sent:ADR\n");
        		}
			attr->sbus_request = SBUS_REQUEST_READ_ADR;
			return(send_dma(attr, attr->sbus_request));
		case BCR:
			
			if(attr->debug_level >= DSP_SHOW_COMMANDS)
        		{
        			if (attr->print_ptr != NULL)
              			attr->print_ptr("\nCommand sent:BCR\n");
         			else
              			fprintf(stdout, "\nCommand sent:BCR\n");
        		}
			attr->sbus_request = SBUS_REQUEST_READ_BCR;
			return(send_dma(attr, attr->sbus_request));
		case Reset:
			send_reset(attr);
			return(DSP_OK);


		/* timing board commands */
		case ABR:
                        send_abr(attr, board_id);
                        return(dsp_get_reply(attr, board_id, DON,
                                             DEFAULT_TIMEOUT));
		case CLR:
			send_clr(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case HGN:
			send_hgn(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case IDL:
			send_idl(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case LGN:
			send_lgn(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case RAD:
			send_rad(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case RDC:
			send_rdc(attr, board_id);
			directory = va_arg(ap,int);
			filename  = va_arg(ap,int);
			ext = va_arg(ap,int);
			numbytes = va_arg(ap,int);
			msecs = va_arg(ap,int);
			image_transfer(attr,board_id,directory,filename,ext,
				numbytes,msecs);
			return(1);
		case SBV:
			send_sbv(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case STP:
			send_stp(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case TCK:
			send_tck(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case TDC:
			send_tdc(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));

		/* utility board commands */
		case AEX:
			send_aex(attr, board_id); 
			board_id = (board_id | 0x000100);
			dsp_get_reply(attr, board_id, ABR, DEFAULT_TIMEOUT);
			board_id = (board_id & 0xfff0ff);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case CSH:
			send_csh(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case OSH:
			send_osh(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case PEX:
			send_pex(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case CON:
			send_con(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case REX:
			send_rex(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     DEFAULT_TIMEOUT));
		case SEX:
			clear_image_buf(MAX_EXPOSURE_BYTES/4);
			attr->first_done = FALSE;
			directory = va_arg(ap,int);
			if (directory == NULL) {
			  send_sex(attr,board_id);
                          return(dsp_get_reply(attr,board_id,
					DON,DEFAULT_TIMEOUT));
			}
			else{
                        filename  = va_arg(ap,int);
                        ext = va_arg(ap,int);
                        numbytes = va_arg(ap,int);
                        msecs = va_arg(ap,int);
                        return(image_transfer(attr, board_id, directory,
                                              filename,ext,numbytes,msecs));
			}
		case RRR:
                        clear_image_buf(MAX_EXPOSURE_BYTES/4);
                        attr->first_done = FALSE;
                        directory = va_arg(ap,int);
                        if (directory == NULL) {
                          send_rrr(attr,board_id);
                          return(dsp_get_reply(attr,board_id,
					DON,DEFAULT_TIMEOUT));
                        }
                        else {
                          filename  = va_arg(ap,int);
                          ext = va_arg(ap,int);
                          numbytes = va_arg(ap,int);
                          msecs = va_arg(ap,int);
                          return(image_transfer(attr, board_id, directory, 
                                                filename,ext,numbytes,msecs));
                        }
		case SYR:
			send_syr(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
						DEFAULT_TIMEOUT));
		case TAD:
			send_tad(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
						DEFAULT_TIMEOUT));
		case TDA:
			send_tda(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
					     	DEFAULT_TIMEOUT));
		case TDG:
			send_tdg(attr, board_id);
			return(dsp_get_reply(attr, board_id, DON,
						DEFAULT_TIMEOUT));
		default:
			fprintf(stderr,
				"libdsp.c:  Unknown DSP command %d\n",
				command);
			return(DSP_ERROR);
	}
}


#ifdef __STDC__
static void clear_reply_mem(int size)
#else /* K&R C */
static void
clear_reply_mem(size)
int		size;
#endif	/* __STDC__ */
{
	int	i;

	for (i=0; i<size; i++) 
		dat.uin[i] = 0;

	return;
}

#ifdef __STDC__
void disp_reply_mem()
#else	/* K&R C */
void
disp_reply_mem()
#endif	/* __STDC__ */
{
	int i;

	for(i=0;i<127;i++)
		message_write(" %d %x \n",dat.uin[i],dat.uin[i]);

	return;
}


#ifdef __STDC__
int send_command(...)
#else /* K&R C */
static int
send_command(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list         ap;
	DSP_ATTR	*attr;
	int		element,
			byteshift,
			argument,
			i;
	char		ch;

	va_start(ap,DSP_ATTR *);
	attr = va_arg(ap, DSP_ATTR *);

	if (attr->abort){
		return DSP_OK;
	}
	

	clear_reply_mem(SBUS_BUFFER_SIZE);
	element = 0;

	argument = va_arg(ap, int);
	if(argument != DSP_END && attr->debug_level >= DSP_SHOW_COMMANDS)
	{
	if (attr->print_ptr != NULL)
              attr->print_ptr("\nCommand sent:");
         else
              fprintf(stdout, "\nCommand sent:");
	}
	while (argument != DSP_END) {
		if (attr->debug_level >= DSP_SHOW_COMMANDS) 
		{
			 if(element==1){
/* This one shows the actual values of the commands instead of the
   ASCII values if the "show commands" is chosen from the properties
   for debugging. */
                                for(i=0;i<3;i++){
                                ch=argument >> 16-i*8;
                                 if (attr->print_ptr != NULL)
                                attr->print_ptr("%c", ch);
                                else
                                fprintf(stdout, "%c", argument);
                                }
                                attr->print_ptr(" ");
                        }
			else if((element == 0) && (argument == 0))
                        {
                                if (attr->print_ptr != NULL)
                                attr->print_ptr("RST\n");
                                else
                                fprintf(stdout, "RST\n");
             			if (attr->print_ptr != NULL)
                     			attr->print_ptr("Returned value: DON\n");
             			else
                     			fprintf(stdout, "Returned value: DON\n");
                        }
			else
			if (attr->print_ptr != NULL)
				attr->print_ptr("%x  ", argument);
			else
				fprintf(stdout, "%x  ", argument);
		}
		if (attr->debug_level != DSP_DO_NOTHING)
		  dat.uin[element] = argument;
		element++;
		argument = va_arg(ap, int);
	}
	if (attr->debug_level != DSP_DO_NOTHING)
	{
		byteshift = (element * 4);
		byteshift <<= 16;
		attr->sbus_request = (byteshift | attr->sbus_request);
		i = ioctl(attr->sbus_fd, attr->sbus_request, reply_arg);
		if (i == -1)
		{
			if (attr->print_ptr != NULL)
				attr->print_ptr("IOCTL() returned error");
			else
				fprintf(stdout, "IOCTL() returned error");
		}
		attr->sbus_request = DEFAULT_SBUS_REQUEST;
	}
	if (attr->debug_level >= DSP_SHOW_COMMANDS) 
	  if (attr->print_ptr != NULL)
	    attr->print_ptr("\n");
	  else
	    fprintf(stdout, "\n");

}


#ifdef	SBUSTOOL
#ifdef	__STDC__
void send_reset(DSP_ATTR *attr)
#else	/* K&R C */
void send_reset(attr)
DSP_ATTR *attr;
#endif	/* __STDC__ */
{
	int	i, data;
	attr->sbus_request = SBUS_RESET;
	send_command(attr, DSP_END);
	send_ww(attr, 0);
}

#ifdef	__STDC__
int send_ww(DSP_ATTR *attr, int data)
#else	/* K&R C */
int send_ww(attr, data)
DSP_ATTR *attr;
int data;
#endif	/* __STDC__ */
{
	attr->sbus_request = SBUS_REQUEST_WORD_WIDTH;
	send_command(attr, data, DSP_END);
}

#ifdef	__STDC__
int send_dma(DSP_ATTR *attr)
#else	/* K&R C */
int send_dma(attr)
DSP_ATTR *attr;
#endif	/* K&R C */
{
	send_command(attr, DSP_END);
	if(attr->debug_level >= DSP_SHOW_COMMANDS)
        {
            if (attr->print_ptr != NULL)
                 attr->print_ptr("Returned value: %x\n",dat.uin[0]);
            else
            fprintf(stdout, "Returned value: %x\n",dat.uin[0]);
        }
	return (dat.uin[0]);
}
#else	/* VMETOOL  */

#ifdef __STDC__
void send_boot(DSP_ATTR *attr, int data)
#else /* K&R C */
void
send_boot(attr, data)
DSP_ATTR        *attr;
int		data;
#endif	/* __STDC__ */
{
	int	*send_addr;
	signal(SIGBUS, sigbus_boot);	/* set bus error to trap function */
	send_addr = attr->send_addr + 1;
	if (attr->abort)
		return;
	if (attr->debug_level >= DSP_SHOW_COMMANDS) 
	{
		if (attr->print_ptr != NULL)
			attr->print_ptr("Boot\n");
		else
			fprintf(stdout, "Boot\n");
	}
	/*if (attr->debug_level != DSP_DO_NOTHING)
		 *send_addr = data; */
}

#ifdef __STDC__
void sigbus_boot(int n)
#else /* K&R C */
void
sigbus_boot()
#endif	/* __STDC__ */
{
	sleep(10000);		 /* sleep for 10000 microseconds */
	signal(SIGBUS, SIG_DFL); /* set bus error back to system default */
	return;			 /* ignore bus error, continue with send_boot */
}


#ifdef __STDC__
int send_sra(DSP_ATTR *attr, int board_id, int address)
#else /* K&R C */
int
send_sra(attr, board_id, address)
DSP_ATTR	*attr;
int		board_id;
int		address;
#endif	/* __STDC__ */
{
	int	board_header, addr_low, addr_high;

	board_header = (board_id << 8) + 4;
	addr_high = (address & HIGH_MASK) >> 16;
	addr_low = (address & LOW_MASK);

	attr->rec_offset = 0;

	send_command(attr, board_header, SRA, addr_high, addr_low, DSP_END);
}

#endif	/* SBUSTOOL */

#ifdef __STDC__
int send_wrm(DSP_ATTR *attr, int board_id, char mem_space,
	     int address, int data)
#else /* K&R C */
int
send_wrm(attr, board_id, mem_space, address, data)
DSP_ATTR	*attr;
int		board_id;
char		mem_space;
int		address;
int		data;
#endif	/* __STDC__ */
{

	int	board_header, mem_mask;
	board_header = (board_id << 8) + 4;

	switch (mem_space) {
		case P:
			mem_mask = P_MASK;
			break;
		case X:
			mem_mask = X_MASK;
			break;
		case Y:
			mem_mask = Y_MASK;
			break;
	}

	send_command(attr,board_header, WRM, address | mem_mask, data, DSP_END);
}

/*int send_set(DSP_ATTR *attr, int board_id, int data)
{
	int board_header;

	board_header = (board_id << 8) + 3;
	send_command(attr,board_header,SET,data,DSP_END);
}*/

#ifdef __STDC__
int send_rdm(DSP_ATTR *attr, int board_id, char mem_space, int address)
#else /* K&R C */
int
send_rdm(attr, board_id, mem_space, address)
DSP_ATTR	*attr;
int		board_id;
char		mem_space;
int		address;
#endif	/* __STDC__ */
{

	int	board_header, mem_mask;

	board_header = (board_id << 8) + 3;

	switch (mem_space) {
		case P:
			mem_mask = P_MASK;
			break;
		case X:
			mem_mask = X_MASK;
			break;
		case Y:
			mem_mask = Y_MASK;
			break;
	}

	send_command(attr, board_header, RDM, address | mem_mask, DSP_END);
}

#ifdef __STDC__
int send_lda(DSP_ATTR *attr, int board_id, int data)
#else /* K&R C */
int
send_lda(attr, board_id, data)
DSP_ATTR	*attr;
int		board_id;
int		data;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 3;

	send_command(attr, board_header, LDA, data, DSP_END);
}

#ifdef __STDC__
int send_sex(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_sex(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

        attr->sbus_request = SBUS_REQUEST_TIMEOUT;
        send_command(attr,75,DSP_END);

	send_command(attr, board_header, SEX, DSP_END);
}

#ifdef __STDC__
int send_rrr(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_rrr(attr, board_id)
DSP_ATTR        *attr;
int             board_id;
#endif  /* __STDC__ */
{

        int     board_header;
        board_header = (board_id << 8) + 2;

        attr->sbus_request = SBUS_REQUEST_TIMEOUT;
        send_command(attr,75,DSP_END);

        send_command(attr, board_header, RRR,DSP_END);
}

#ifdef __STDC__
int send_pon(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_con(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;
	attr->sbus_request = SBUS_REQUEST_TIMEOUT;
	send_command(attr,1500,DSP_END);

	send_command(attr, board_header, PON, DSP_END);
}

 
int send_con(DSP_ATTR *attr, int board_id)
{
 
        int     board_header;

        board_header = (board_id << 8) + 2;
        attr->sbus_request = SBUS_REQUEST_TIMEOUT;
        send_command(attr,1500,DSP_END);
 
        send_command(attr, board_header, CON, DSP_END);
}

#ifdef __STDC__
int send_hgn(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_hgn(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, HGN, DSP_END);
}

#ifdef __STDC__
int send_lgn(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_lgn(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, LGN, DSP_END);
}

#ifdef __STDC__
int send_rad(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_rad(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, RAD, DSP_END);
}

#ifdef __STDC__
int send_idl(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_idl(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, IDL, DSP_END);
}

#ifdef __STDC__
int send_stp(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_stp(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, STP, DSP_END);
}


#ifdef __STDC__
int send_clr(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_clr(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, CLR, DSP_END);
}


#ifdef __STDC__
int send_osh(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_osh(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, OSH, DSP_END);
}


#ifdef __STDC__
int send_csh(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_csh(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, CSH, DSP_END);
}


#ifdef __STDC__
int send_rst(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_rst(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, RST, DSP_END);
}


#ifdef __STDC__
int send_abr(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_abr(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, ABR, DSP_END);
}


#ifndef	SBUSTOOL
#ifdef __STDC__
int send_irq(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_irq(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, IRQ, DSP_END);
}
#endif	/* SBUSTOOL */

#ifdef __STDC__
int send_rdc(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_rdc(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, RDC, DSP_END);
}


#ifdef __STDC__
int send_sbv(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_sbv(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, SBV, DSP_END);
}


#ifdef __STDC__
int send_tck(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_tck(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, TCK, DSP_END);
}


#ifdef __STDC__
int send_tdc(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_tdc(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, TDC, DSP_END);
}


#ifdef __STDC__
int send_aex(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_aex(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, AEX, DSP_END);
}

#ifdef __STDC__
int send_pex(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_pex(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, PEX, DSP_END);
}

#ifdef __STDC__
int send_rex(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_rex(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, REX, DSP_END);
}

#ifdef __STDC__
int send_syr(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_syr(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, SYR, DSP_END);
}

#ifdef __STDC__
int send_tad(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_tad(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, TAD, DSP_END);
}

#ifdef __STDC__
int send_tda(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_tda(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, TDA, DSP_END);
}

#ifdef __STDC__
int send_tdg(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_tdg(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, TDG, DSP_END);
}


#ifdef __STDC__
int send_tdl(DSP_ATTR *attr, int board_id, int data)
#else /* K&R C */
int
send_tdl(attr, board_id, data)
DSP_ATTR	*attr;
int		board_id;
int		data;
#endif	/* __STDC__ */
{

	int	board_header;
	board_header = (board_id << 8) + 3;

	send_command(attr, board_header, TDL, data, DSP_END);
}

#ifndef	SBUSTOOL
#ifdef	IRTOOL

#ifdef __STDC__
int send_mm1(DSP_ATTR *attr, int board_id, int data)
#else /* K&R C */
int
send_mm1(attr, board_id, data)
DSP_ATTR	*attr;
int		board_id;
int		data;
#endif	/* __STDC__ */
{
	int board_header;
	board_header = (board_id << 8) + 3;

	send_command(attr, board_header, MM1, data, DSP_END);
}


#ifdef __STDC__
int send_mh1(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_mh1(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{
	int board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, MH1, DSP_END);
}


#ifdef __STDC__
int send_mm2(DSP_ATTR *attr, int board_id, int data)
#else /* K&R C */
int
send_mm2(attr, board_id, data)
DSP_ATTR	*attr;
int		board_id;
int		data;
#endif	/* __STDC__ */
{
	int board_header;
	board_header = (board_id << 8) + 3;

	send_command(attr, board_header, MM2, data, DSP_END);
}


#ifdef __STDC__
int send_mh2(DSP_ATTR *attr, int board_id)
#else /* K&R C */
int
send_mh2(attr, board_id)
DSP_ATTR	*attr;
int		board_id;
#endif	/* __STDC__ */
{
	int board_header;
	board_header = (board_id << 8) + 2;

	send_command(attr, board_header, MH2, DSP_END);
}


#endif	/* IRTOOL */
#endif	/* SBUSTOOL */

int dsp_get_reply(DSP_ATTR *attr, int board_id,
		  int expected_reply, int timeout)
{
	int	reply_header, reply_action;
	int	value, i=0, j=0, done=0;
	int 	local_errno;

	clear_reply_mem(SBUS_BUFFER_SIZE);

	attr->last_header = attr->last_reply = 0;
	reply_action = attr->reply_action;

	
	if (reply_action == REPLY_NO_WAIT)
	  return DSP_WAITING;
	
	reply_header = (board_id << 16) + 2;

	if (attr->debug_level == DSP_DO_NOTHING)
	  return DSP_OK;

	while ((!done) && (attr->abort == FALSE)) {
          value = read(attr->sbus_fd, reply_arg, SBUS_REPLY_SIZE);
	  local_errno = errno;
	  done = 1;
	  if (reply_action == CALL_REPLY_FUNC)
            attr->reply_func_ptr();
	}
	if (attr->abort)
	  return DSP_ABORTED;

	attr->last_header = (dat.uin[0] & REPLY_MASK);
	attr->last_reply = (dat.uin[1] & REPLY_MASK);

	if (attr->last_header != reply_header) {
	  if (dsp_bad_header_count <= MAX_DSP_BAD_REPLY_HEADERS) {
	    dsp_errno = DSP_BAD_REPLY_HEADER;
	    if (attr->debug_level >= DSP_SHOW_ERRORS)
	      if (attr->print_ptr != NULL) {
		attr->print_ptr("\n	*********** BAD REPLY HEADER\n");
		attr->print_ptr("	expected header: %08x\n", reply_header);
		attr->print_ptr("	actual header:   %08x\n", attr->last_header);
		attr->print_ptr("	actual reply:    %08x\n", attr->last_reply);
	      }
	      else {
		fprintf(stdout, "\n	*********** BAD REPLY HEADER\n");
		fprintf(stdout, "	expected header: %08x\n", reply_header);
		fprintf(stdout, "	actual header:   %08x\n", attr->last_header);
		fprintf(stdout, "	actual reply:    %08x\n", attr->last_reply);
	      }
	      report_errno(attr,local_errno);
	      return DSP_ERROR;
	  }
	}


	if (expected_reply == DSP_ACTUAL_VALUE) {
		if (attr->debug_level >= DSP_SHOW_COMMANDS)
			if (attr->print_ptr != NULL)
				attr->print_ptr("\n	returned value: %08x\n", attr->last_reply);
			else
				fprintf(stdout, "\n	returned value: %08x\n", attr->last_reply);
		return attr->last_reply;
	}
	else if(attr->last_reply == ERR) {
		if (attr->debug_level == DSP_SHOW_ERRORS) 
			if (attr->print_ptr != NULL)
				attr->print_ptr("\n***ERR\n");
			else
				fprintf(stdout, "\n***ERR\n");
		dsp_errno = DSP_REPLY_ERROR;
#ifdef   SBUSTOOL
		report_errno(attr,local_errno);
#endif   /* SBUSTOOL */
		return DSP_ERROR;
	}
	else if (attr->last_reply == expected_reply) {
		if (attr->debug_level >= DSP_SHOW_COMMANDS)
			if (attr->print_ptr != NULL)
				attr->print_ptr("\n     returned value: %08x\n",attr->last_reply);
			else
				fprintf(stdout, "\n     returned value: %08x\n",attr->last_reply);
		return DSP_OK;
	}
	else {
		if (attr->debug_level == DSP_SHOW_ERRORS) 
			if (attr->print_ptr != NULL) {
				attr->print_ptr("\n	*********** UNKNOWN reply\n");
				attr->print_ptr("	expected reply:  %08x\n", expected_reply);
				attr->print_ptr("	actual reply:    %08x\n", attr->last_reply);
			}
			else {
				fprintf(stdout, "	*********** UNKNOWN reply\n");
				fprintf(stdout, "	expected reply:  %08x\n", expected_reply);
				fprintf(stdout, "	actual reply:    %08x\n", attr->last_reply);
			}
#ifdef	SBUSTOOL
		report_errno(attr,local_errno);
#endif  /* SBUSTOOL */	
		return DSP_ERROR;
	}
}


#ifdef __STDC__
int dsp_poll(DSP_ATTR *attr, int board_id, int expected_reply)
#else /* K&R C */
int
dsp_poll(attr, board_id, expected_reply)
DSP_ATTR	*attr;
int		board_id;
int		expected_reply;
#endif	/* __STDC__ */
{
#ifdef	SBUSTOOL
	int	reply_header, value;
	int	reply, which_board;

	reply_header = (board_id << 16) + 2;

	which_board = dat.uin[0] & REPLY_MASK;
	reply = dat.uin[1] & REPLY_MASK;

#else	/* VMETOOL  */
	int	reply_header;
	int	*base_reply_addr, *reply_addr;
	int	reply, which_board;

	reply_header = (board_id << 16) + 2;
	base_reply_addr = attr->rec_addr + attr->rec_offset;
	reply_addr = base_reply_addr + 1;

	reply = *reply_addr & REPLY_MASK;
	which_board = *base_reply_addr & REPLY_MASK;

#endif	/* SBUSTOOL */

	if (reply == expected_reply) {
		if (which_board == reply_header)
			return DSP_OK;
		else {
			dsp_errno = DSP_BAD_REPLY_HEADER;
			return DSP_ERROR;
		}
	}
	else if (reply == ERR) {
		dsp_errno = DSP_REPLY_ERROR;
		return DSP_ERROR;
	}
	else
		return DSP_WAITING;
}


#ifndef	SBUSTOOL	/* VMETOOL  */
#ifdef __STDC__
void dsp_update_offset(DSP_ATTR *attr)
#else /* K&R C */
void
dsp_update_offset(attr)
DSP_ATTR	*attr;
#endif	/* __STDC__ */
{
	attr->rec_offset = (attr->rec_offset+2) % (attr->vme_rec_size / sizeof(int));
}
#endif	/* SBUSTOOL */


#ifdef __STDC__
int dsp_download(DSP_ATTR *attr, int board, char *filename)
#else /* K&R C */
int
dsp_download(attr, board, filename)
DSP_ATTR	*attr;
int		board;
char		*filename;
#endif	/* __STDC__ */
{

	FILE	*download_fp;
	int	finished, board_id, addr,ret_value,flag = 0;
	char	buf[255], addr_type;

	if ((download_fp = fopen(filename, "r")) == NULL) {
		dsp_errno = DSP_DOWNLOAD_OPEN_ERROR;
		return DSP_ERROR;
	}

	if ((board_id = dsp_get_download_type(download_fp)) == DSP_ERROR) {
		dsp_errno = DSP_DOWNLOAD_FILE_ERROR;
		return DSP_ERROR;
	}

	if (board != board_id) {
		dsp_errno = DSP_DOWNLOAD_TYPE_ERROR;
		return DSP_ERROR;
	}

	finished = FALSE;
	while ((!finished) && (!attr->abort)) {
		dsp_read_line(download_fp, buf);
		if (strncmp(buf, "_END", 4) == 0) {
			finished = TRUE;
		}
		else if (sscanf(buf, "_DATA %c %x", &addr_type, &addr) == 2) {
			if (addr < DSP_DOWNLOAD_ADDR_MAX) {
				ret_value = dsp_process_data(attr, download_fp, board_id, addr_type, addr);
				if(ret_value == 1)
					flag = 1;
			}
		}
	}
	fclose(download_fp);
	if(flag == 1)
		return(DSP_ERROR);
	else if(flag == 0)
		return(DSP_OK);

}


#ifdef __STDC__
static int dsp_read_line(FILE *fp, char *buf)
#else /* K&R C */
static int
dsp_read_line(fp, buf)
FILE* fp;
char* buf;
#endif	/* __STDC__ */
{
	if (fgets(buf, 80, fp) == NULL) {
		perror("error on input");
		exit(1);
	}
}


#ifdef __STDC__
static int dsp_get_download_type(FILE *fp)
#else /* K&R C */
static int
dsp_get_download_type(fp)
FILE    *fp;
#endif	/* __STDC__ */
{

	char    *cp, buf[255];
	int     id;

	dsp_read_line(fp, buf);
	if (strncmp(buf, "_START", 6) != 0) {
		return DSP_ERROR;
	}

	cp = buf+7;
	switch (*cp)
	{
#		ifndef	SBUSTOOL
		case 'V':
			id = VME_ID;
			break;
#		endif	/* SBUSTOOL */
		case 'T':
			id = TIM_ID;
			break;
		case 'U':
			id = UTIL_ID;
			break;
		default:
			return DSP_ERROR;
	}

	return(id);
}

#ifdef __STDC__
static int dsp_process_data(DSP_ATTR *attr, FILE *download_fp, 
			    int board_id, char type, int addr)
#else /* K&R C */
static int
dsp_process_data(attr, download_fp, board_id, type, addr)
DSP_ATTR	*attr;
FILE		*download_fp;
int		board_id;
char		type;
int		addr;
#endif	/* __STDC__ */
{
 
        int     finished, value, i,flag = 0,ret_value;
        char    c;
 
        finished = FALSE;
        while ((!finished) && (!attr->abort)) {
                while ((c = getc(download_fp)) == 32);
                if (c == '_')  {
                        ungetc(c, download_fp);
                        finished = TRUE;
                }
                else if ((c != 10) && (c != 13)){
                        ungetc(c, download_fp);
                        fscanf(download_fp, "%x", &value);
                        dsp_command(attr, board_id, WRM, type, addr, value);
			if(ret_value == DSP_ERROR)
				flag = 1;
                        addr++;
                }
        }
	return(flag);
}

#ifdef __STDC__
void dsp_perror(char *s)
#else /* K&R C */
void
dsp_perror(s)
char	*s;
#endif	/* __STDC__ */
{
	fprintf(stderr, "%s:  %s\n", s, dsp_errlist[dsp_errno]);
}


int image_transfer(...)
{
	va_list ap;
	DSP_ATTR *attr;
        int     numbytes,i, j, imagesave,value, exp_fd;
	int	board_id,rem_msecs,tot_msecs, dspmsecs, check_time_temp=0;
	int	reply_action, ww = 0;
	int 	local_errno;
	char    *directory, *filename;
        char    buffer[MAX_PATH_NAME];
	int     *base_reply_addr, *reply_addr;
	int 	*send_addr;
	int     reply_header;
        char    *image_1;
	FILE	*exp_infp;
	unsigned short *first_image, *second_image;

        va_start(ap,NULL);
        attr = va_arg(ap, DSP_ATTR *);
        board_id = va_arg(ap, int);
        directory = (char *)va_arg(ap, int);
        filename = (char *)va_arg(ap, int);
        imagesave = va_arg(ap, int);
        numbytes = va_arg(ap, int);
        tot_msecs = rem_msecs = va_arg(ap,int);;

        exp_byte = (char *)&expdata.ch[0];

	
        sprintf(buffer, "%s/%s", directory, filename);
        if ((exp_fd = open(buffer, O_RDWR | O_CREAT,0660)) == -1)
	{
                printf("\nNot able to open image file\n");
                return(DSP_ERROR);
        }

	reply_action = attr->reply_action;

        if ((exp_infp = fopen(buffer, "r+")) == NULL)
	{
                printf("\nNot able to fopen image file\n");
                return(DSP_ERROR);
        }
        else
	{
		if (!dual_image_mode)	
		  send_sex(attr, board_id);
                else
                  send_rrr(attr, board_id);

		if (dsp_get_reply(attr, board_id, DON, DEFAULT_TIMEOUT) ==
                        DSP_ERROR) {
                  return DSP_ERROR;
                }
                else {

		  	sprintf(buffer, "Reading out CCD...");
	          	message_write("Reading out CCD...\n");
        	  	readout_busy(TRUE);
        	  	write_frame(Expctl->expctl_popup, FRAME_LEFT_FOOTER, buffer);

                  	if (dual_image_mode) 
                        {
                        	attr->sbus_request = SBUS_REQUEST_TIMEOUT;

  	                	if (numbytes<2000000) 
                          	  send_command(attr,200,DSP_END);
                        	else 
                          	  if (numbytes<8000000) 
                            	    send_command(attr,600,DSP_END);
                          	  else 
                            	    send_command(attr,1500,DSP_END);

                    		if (ww != 1) send_ww(attr,1);

                        	value = read(attr->sbus_fd, exp_byte, numbytes);
		        	local_errno = errno;
		        	report_errno(attr,local_errno);
		        	send_ww(attr,0);
		        	image_1 = exp_byte;
		        	exp_byte += numbytes;
	                } 
		
		        if (rem_msecs <= 4000) 
			{
	                  send_ww(attr, 1);
		          ww = 1;
	                  attr->sbus_request = SBUS_REQUEST_TIMEOUT;
		          send_command(attr,1500,DSP_END);
	                }
		        attr->first_done = TRUE;

			while ((rem_msecs > 4000) && (attr->abort == FALSE))
			{
			
		  		sleep(1);
		  		attr->sbus_request = SBUS_REQUEST_TIMEOUT;
		  		send_command(attr,75,DSP_END);
		  		dspmsecs = dsp_command(attr, TIM_ID, RDM, X, 2);
		  		rem_msecs = tot_msecs - dspmsecs;
		  		if (pause_flag)
		    		{
	            			pause_expose();
		    			pause_flag = FALSE;
		    		}
		  		if (abort_flag)
		    		{
		    			abort_expose();
		    			abort_flag = FALSE;
		    		} 
		  		if (time_flag)
		    		{
		    			rem_msecs = change_time(attr,tot_msecs-rem_msecs);
		    			tot_msecs = dspmsecs + rem_msecs;
		    			time_flag = FALSE;
		    		}
		  		if (reply_action == CALL_REPLY_FUNC)
		    		  attr->reply_func_ptr();
		  		sprintf(buffer,"Exposure time remaining (secs): %d",
								(int)rem_msecs/1000);
		  		write_frame(Expctl->expctl_popup, FRAME_LEFT_FOOTER, buffer);
			} /* While */

        		attr->sbus_request = SBUS_REQUEST_TIMEOUT;
        
			if (numbytes<2000000)
              		  send_command(attr,200,DSP_END);
        		else 
                          if (numbytes<8000000)
              		    send_command(attr,600,DSP_END);
        		  else
              		    send_command(attr,1500,DSP_END);
 

			if (attr->abort)
        		{
               			dsp_command(attr, TIM_ID, AEX);
               			return DSP_ABORTED;
        		}
	
			if (ww != 1) send_ww(attr, 1);
        		extr = 0;

        		value = read(attr->sbus_fd, exp_byte, numbytes);
			local_errno = errno;
			report_errno(attr,local_errno);
        		send_ww(attr, 0);

			/* 
			 * Subtraction between the two images.
			 */

        		if (dual_image_mode) 
			{
			  first_image  = &expdata.ush[0];
                          second_image = &expdata.ush[numbytes/2];

	  		  for (j=0; j < (numbytes/2); j++) {
             		     *first_image = *second_image - *first_image;
             		     first_image++;
             		     second_image++;
          		  }
			}

			attr->last_header = (dat.uin[0] & REPLY_MASK);
          		if (imagesave > 0) {
            		  lseek(exp_fd, FITS_HEADER_SIZE, 0);
	    		  write(exp_fd, &expdata.ch[extr], numbytes);
          		}

			fclose(exp_infp);
			close(exp_fd);
			return(DSP_OK);
		} /* else get reply */
	} /* else open file */
} /* image_transfer */

#ifdef	__STDC__
static void clear_image_buf(int size)
#else	/* K&R C */
static void
clear_image_buf(size)
int	size;
#endif	/* __STDC__ */
{
	int	i;

	for (i=0; i<size; i++)
		expdata.uin[i] = 0;

}
#ifdef SBUSTOOL
int     save_image(char *directory, char *filename, int numbytes)
{
        int i,exp_fd;
        FILE *exp_infp;
        char buffer[MAX_PATH_NAME];

        sprintf(buffer, "%s/%s", directory, filename);
        if ((exp_fd = open(buffer, O_RDWR)) == -1) {
                printf("\nNot able to open image file\n");
                return(DSP_ERROR);
        }
 
 
        if ((exp_infp = fopen(buffer, "r+")) == NULL) {
                printf("\nNot able to fopen image file\n");
                return(DSP_ERROR);
        }
 
        lseek(exp_fd, FITS_HEADER_SIZE, 0);
        write(exp_fd, &expdata.ch[0], numbytes);
        fclose(exp_infp);
        close(exp_fd);
}


#ifdef  __STDC__
static void     report_errno(DSP_ATTR *attr,int local_errno)
#else  
static void    report_errno(attr,local_errno)
DSP_ATTR *attr;
int local_errno;
#endif  
{
	switch(local_errno)
	{
		case ENOMEM:
		if (attr->print_ptr != NULL)
			attr->print_ptr("\nAllocation of buffers failed\n");
		else 
			fprintf(stderr,"\nAllocation of buffers failed\n");
		break;
		case ECHILD:
		if (attr->print_ptr != NULL)
			attr->print_ptr("\nAll the image buffers are busy\n");
		else
			fprintf(stderr,"\nAll the image buffers are busy\n");
		break;
		case ENOTEMPTY:
		if (attr->print_ptr != NULL)
                        attr->print_ptr("\nBad interrupt encountered\n");
                else
                        fprintf(stderr,"\nBad interrupt encountered\n");
                break;	
		case ETIME:
		if (attr->print_ptr != NULL)
			attr->print_ptr("\nData transfer time out\n");
		else
			 fprintf(stderr,"\nData transfer time out\n");
		break;
		case EDEADLK:
			attr->print_ptr("\nImage buffer not busy\n");	
			break;
		case ENOLCK:
			attr->print_ptr("\nData transfer error\n");
			break;
		case EPIPE:
			attr->print_ptr("\nCopy error to user space\n");
			break;
		case EPERM:
			attr->print_ptr("Image buffer not done\n");
			break;
		case ENXIO:
			attr->print_ptr("\nThere is no DMA device\n");
			break;
		case ENOTBLK:
                        attr->print_ptr("\nDMA not busy\n");
			break; 
		case EIO:
			attr->print_ptr("\nBad value returned from image transfer\n");
			break;
	}
}
#endif	/* SBUSTOOL */


/*
 * $Log: libdsp.c,v $
 * Revision 1.81  1997/01/03 22:55:30  gumrukcu
 * ***ABDUL***
 * minor changes.
 *
 * Revision 1.80  1997/01/03 22:21:48  gumrukcu
 * ***ABDUL***
 * add timeout.
 *
 * Revision 1.78  1996/11/25 21:24:38  peker
 * Added some new functions and cleared some buggs.
 *
 * Revision 1.77  1996/06/20  22:14:16  peker
 * test
 *
 * Revision 1.76  1996/06/19 19:53:42  peker
 * changed some code for the ANSI C standards...
 *
 * Revision 1.75  1996/06/14 21:15:33  peker
 * fix some bugs
 *
 * Revision 1.72  1996/03/27  20:02:46  gumrukcu
 * EIO is added.
 *
 * Revision 1.71  1996/03/07  21:59:03  gumrukcu
 * test10
 *
 * Revision 1.56  1996/03/07 21:21:02  gumrukcu
 * test keeps going on
 *
 * Revision 1.50  1996/03/07 20:59:16  gumrukcu
 * Just a test to see the syntax error..
 *
 * Revision 1.49  1996/03/07 20:29:38  gumrukcu
 * Error EIO is commented.
 * ./
 * .`
 *
 * Revision 1.48  1996/03/07  00:54:47  gumrukcu
 * Local errno has been changed.
 *
 * Revision 1.46  1996/03/01  04:04:10  gumrukcu
 * Some better error reporting is made.
 *
 * Revision 1.46  1996/03/01  04:04:10  gumrukcu
 * Some better error reporting is made.
 *
 * Revision 1.45  1996/02/29  22:23:04  gumrukcu
 * Spme reportings of the errors are added which are caused in the device driver.
 *
 * Revision 1.44  1996/02/26  19:38:04  gumrukcu
 * Debugging purposed printfs are cleared.
 *
 * Revision 1.43  1996/02/26  19:36:37  gumrukcu
 * some changes in dsp_command function.
 *
 * Revision 1.42  1996/02/21  22:19:08  gumrukcu
 * The
 *
 * Sbus board is reset before sending the TDL.
 *
 * Revision 1.41  1996/02/07  22:07:13  gumrukcu
 * Some format changes for when the show command are chosen.
 *
 * Revision 1.40  1996/02/06  18:52:31  gumrukcu
 * Whne the dsp reply is don, the "returned value:DON" is printed.
 *
 * Revision 1.39  1996/02/02  19:32:31  gumrukcu
 * The initialization of the image buf is comemnted out.
 *
 * Revision 1.38  1996/02/02  19:15:49  gumrukcu
 * The image buffer is initialized to zero in clear_image_bug function.
 *
 * Revision 1.37  1996/01/30  22:59:27  gumrukcu
 * Some changes in image_transfer function.
 *
 * Revision 1.36  1996/01/29  23:29:33  gumrukcu
 * Some changes for the dsp window for the sbus commands and for the
 * readoutccd button in image transfer function.
 *
 * Revision 1.35  1996/01/28  00:43:36  gumrukcu
 * A syntax error is corretcted.
 *
 * Revision 1.34  1996/01/28  00:41:31  gumrukcu
 * Some more properties for the option of show commands.
 *
 * Revision 1.33  1996/01/27  22:30:27  gumrukcu
 * some changes for the show commands property.
 *
 * Revision 1.32  1996/01/27  21:05:15  gumrukcu
 * In the show commands some features are added.
 *
 * Revision 1.31  1996/01/18 01:24:27  gumrukcu
 * Some changes in image_transfer function.
 *
 * Revision 1.30  1996/01/11  02:27:19  gumrukcu
 * Syntax error is corrected.
 *
 * Revision 1.26  1996/01/05  21:19:17  gumrukcu
 * Declaration of dsp_command has been changed.
 *
 * Revision 1.24  1996/01/05 00:27:59  gumrukcu
 * attr = (DSP_ATTR *)va_arg(ap, int) is commented out. Because it is already passed.
 *
 * Revision 1.19  1996/01/05 00:01:21  gumrukcu
 * working on dsp_command function for sloris.
 *
 * Revision 1.18  1996/01/04 04:23:04  gumrukcu
 * declaration of dsp_command is changed.
 *
 * Revision 1.17  1996/01/04  04:01:45  gumrukcu
 * usleep is changed to sleep.
 *
 * Revision 1.16  1996/01/04 03:21:29  gumrukcu
 * declaration of dsp_command is changed.
 *
 * Revision 1.15  1996/01/04 02:52:50  gumrukcu
 * declaration of save_image is commented out.
 *
 * Revision 1.14  1996/01/04 02:50:18  gumrukcu
 * the declaration of save image is changed.
 *
 * Revision 1.13  1996/01/04 01:41:56  gumrukcu
 * save_image is added back again.
 *
 * Revision 1.12  1996/01/03 20:49:25  gumrukcu
 * Just a test.
 *
 * Revision 1.11  1996/01/03 20:22:36  gumrukcu
 * image_transfer is isolated for SBUSTOOL.
 *
 * Revision 1.10  1996/01/03  01:11:38  gumrukcu
 * SOme corrections were made for passing the arguments to the functions.
 * I hope this is the last one.
 *
 * Revision 1.9  1996/01/02  23:16:25  gumrukcu
 * some changes are made in image_transfer because of new additions in the expctl window.
 *
 * Revision 1.8  1995/12/22  23:22:21  gumrukcu
 * syntax error is corrected from the previous version.
 *
 * Revision 1.7  1995/12/22  23:17:53  gumrukcu
 * save_image is added in.
 *
 * Revision 1.6  1995/12/15  00:45:17  gumrukcu
 * save_image function is added.
 *
 * Revision 1.5  1995/12/12  22:33:22  tjk
 * Merged SBUS and VME versions
 *
 * Revision 1.4  1995/08/23  00:59:08  moore
 * ANSI conversion
 *
 * Revision 1.3  1995/08/23 00:09:00  tjk
 * ANSI conversion...
 *
 * Revision 1.2  1995/08/10  16:16:12  tjk
 * Modified for IRTOOL
 *
 * Revision 1.1  1995/02/10  03:37:43  tjk
 * Initial revision
 *
 */
  
