/* $Header: /rosie/src/ccd_utilities/src/libfits/RCS/libfits.c,v 1.14 1996/06/15 00:16:36 tjk Exp $
 */

#include 	<stdio.h>
#include	<fcntl.h>
#include        <sys/types.h>
#include        <sys/mman.h>
#include 	<varargs.h>
#include	<unistd.h>
#include	<time.h>
#include	"fits.h"

int		fits_errno;

char	*fits_errlist[] =
{
	"no error",
	"error opening FITS file",
	"error closing FITS file",
};

/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
FIOS fios[] = {
        { FITS_AIRMASS,         "AIRMASS =" },
        { FITS_BITPIX,		"BITPIX  =" },
        { FITS_BLANK,		"         " },
        { FITS_COMMENT,         "COMMENT  " },
        { FITS_DATE_OBS,	"DATE-OBS=" },
        { FITS_DEC,             "DEC     =" },
        { FITS_DETECTOR,	"DETECTOR=" },
        { FITS_END,		"END      " },
        { FITS_EPOCH,		"EPOCH   =" },
        { FITS_EXPTIME,         "EXPTIME =" },
        { FITS_EXPTYPE,         "EXPTYPE =" },
        { FITS_FILTER,          "FILTER  =" },
        { FITS_HISTORY,         "HISTORY  " },
        { FITS_INSTRUMENT,      "INSTRUME=" },
        { FITS_IRAFNAME,        "IRAFNAME=" },
        { FITS_ISTART,          "ISTART  =" },
        { FITS_JSTART,          "JSTART  =" },
        { FITS_NAXIS,           "NAXIS   =" },
        { FITS_NAXIS1,          "NAXIS1  =" },
        { FITS_NAXIS2,          "NAXIS2  =" },
        { FITS_OBJECT,          "OBJECT  =" },
        { FITS_OBSERVATORY,	"OBSERVAT=" },
        { FITS_OBSERVER,        "OBSERVER=" },
        { FITS_POSTSCAN,	"POSTSCAN=" },
        { FITS_PRESCAN,         "PRESCAN =" },
        { FITS_RA,              "RA      =" },
        { FITS_SIDEREAL_TIME,	"ST      =" },
        { FITS_SIMPLE,          "SIMPLE  =" },
        { FITS_TELESCOPE,       "TELESCOP=" },
        { FITS_UTSTART,         "UTSTART =" }
};


#ifdef	__STDC__
fits_opaque *fits_create(void)
#else	/* K&R C */
fits_opaque
*fits_create()
#endif	__STDC__
{
	int i;

	static FITS_OBJ *obj;

	/* These variables are for setting the date and time */
	char		gmt[80], date[80];
	struct tm	*gm_time;
	time_t		tp;


	/* Initialize the date and time */
	time(&tp);
	gm_time = gmtime(&tp);

	strftime(date, 70, "%d/%m/%y", gm_time);
	strftime(gmt, 70, "%T", gm_time);

	if ((obj = (FITS_OBJ *)malloc(sizeof(FITS_OBJ))) != NULL)
	{
#		ifndef	SBUSTOOL
		/* Set VME image address to NULL */
		obj->image_addr = NULL;
#		endif	/* SBUSTOOL */

		if (getcwd(obj->directory, MAX_DIRECTORY_SIZE) == NULL)
			return NULL;
		strncpy(obj->filename, DEFAULT_FITS_FILENAME,
			strlen(DEFAULT_FITS_FILENAME)+1);

		/* FITS Tags */
/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
		obj->airmass = DEFAULT_FITS_AIRMASS;
		obj->bitpix = DEFAULT_FITS_BITPIX;
		strncpy(obj->date_observation, date,
			strlen(date)+1);
		strncpy(obj->dec, DEFAULT_FITS_DEC, strlen(DEFAULT_FITS_DEC)+1);
		strncpy(obj->detector, DEFAULT_FITS_DETECTOR,
			strlen(DEFAULT_FITS_DETECTOR)+1);
		obj->epoch = DEFAULT_FITS_EPOCH;
		obj->exptime = DEFAULT_FITS_EXPTIME;
		strncpy(obj->exptype, DEFAULT_FITS_EXPTYPE,
			strlen(DEFAULT_FITS_EXPTYPE)+1);
		strncpy(obj->filter, DEFAULT_FITS_FILTER,
			strlen(DEFAULT_FITS_FILTER)+1);
		strncpy(obj->instrument, DEFAULT_FITS_INSTRUMENT,
			strlen(DEFAULT_FITS_INSTRUMENT)+1);
		strncpy(obj->iraf_name, DEFAULT_FITS_IRAFNAME,
			strlen(DEFAULT_FITS_IRAFNAME)+1);
		obj->istart = DEFAULT_FITS_ISTART;
		obj->jstart = DEFAULT_FITS_JSTART;
		obj->naxis = DEFAULT_FITS_NAXIS;
		obj->naxis1 = DEFAULT_FITS_NAXIS1;
		obj->naxis2 = DEFAULT_FITS_NAXIS2;
		strncpy(obj->object, DEFAULT_FITS_OBJECT,
			strlen(DEFAULT_FITS_OBJECT)+1);
		strncpy(obj->observatory, DEFAULT_FITS_OBSERVATORY,
			strlen(DEFAULT_FITS_OBSERVATORY)+1);
		strncpy(obj->observer, DEFAULT_FITS_OBSERVER,
			strlen(DEFAULT_FITS_OBSERVER)+1);
		obj->postscan = DEFAULT_FITS_POSTSCAN;
		obj->prescan = DEFAULT_FITS_PRESCAN;
		strncpy(obj->ra, DEFAULT_FITS_RA, strlen(DEFAULT_FITS_RA)+1);
		strncpy(obj->sidereal_time, DEFAULT_FITS_SIDEREAL_TIME,
			strlen(DEFAULT_FITS_SIDEREAL_TIME)+1);
		obj->simple = DEFAULT_FITS_SIMPLE;
		strncpy(obj->telescope, DEFAULT_FITS_TELESCOPE,
			strlen(DEFAULT_FITS_TELESCOPE)+1);
		strncpy(obj->ut_start, gmt,
			strlen(gmt)+1);

		/* Initialize user keywords */
		for (i = 0; i < MAX_USER_KEYWORDS; i++)
		{
			strncpy(obj->user_keywords[i],
				DEFAULT_FITS_KEYWORD,
				strlen(DEFAULT_FITS_KEYWORD)+1);

			strncpy(obj->user_keyword_values[i],
				DEFAULT_FITS_KEYWORD_VALUE,
				strlen(DEFAULT_FITS_KEYWORD_VALUE)+1);
		}
	}
	return (fits_opaque *)obj;
}


#ifdef	__STDC__
static void	clear_image_mem(FITS_OBJ *obj, int size)
#else	/* K&R C */
static void	clear_image_mem(obj, size)
FITS_OBJ	*obj;
int		size;
#endif
{
	int	i;

	for (i = 0; i < size; i++)
		obj->image_addr[i] = 0;

	return;
}


#ifdef	__STDC__
fits_opaque fits_get(...)
#else	/* K&R C */
fits_opaque
fits_get(va_alist)
va_dcl
#endif	__STDC__
{
	va_list		ap;
	FITS_OBJ	*obj;
	int		modifier, value;
	static char	buffer[MAX_BUF_SIZE+1];
	char		char_value;
	double		double_value;

	va_start(ap);
	obj = va_arg(ap, FITS_OBJ *);
	modifier = va_arg(ap, int);

	switch (modifier) {

		case FITS_FD:
			value = obj->fd;
			return (fits_opaque)value;
		case FITS_IMAGE_ADDR:
			return (fits_opaque)(obj->image_addr);
		case FITS_DIRECTORY:
			strncpy(buffer, obj->directory,
				strlen(obj->directory)+1);
			return (fits_opaque)buffer;
		case FITS_FILENAME:
			strncpy(buffer, obj->filename, strlen(obj->filename)+1);
			return (fits_opaque)buffer;

		/* FITS Tags */
/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
		case FITS_AIRMASS:
			double_value = (double) obj->airmass;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf, %lf\n", modifier, obj->airmass,
	double_value);
#endif	DEBUG_FITS_HEADER
			return (double) double_value;
		case FITS_BITPIX:
			value = obj->bitpix; 
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->bitpix);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_DATE_OBS:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, buffer);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->date_observation;
		case FITS_DEC:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->dec);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->dec;
		case FITS_DETECTOR:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->detector);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->detector;
		case FITS_EPOCH:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf\n", modifier, obj->epoch);
#endif	DEBUG_FITS_HEADER
			return (float)obj->epoch;
		case FITS_EXPTIME:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf\n", modifier, obj->exptime);
#endif	DEBUG_FITS_HEADER
			return (float)obj->exptime;
		case FITS_EXPTYPE:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->exptype);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->exptype;
		case FITS_FILTER:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->filter);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->filter;
		case FITS_INSTRUMENT:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->instrument);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->instrument;
		case FITS_IRAFNAME:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->iraf_name);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->iraf_name;
		case FITS_ISTART:
			value = obj->istart;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->istart);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_JSTART:
			value = obj->jstart;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->jstart);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_NAXIS:
			value = obj->naxis;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_NAXIS1:
			value = obj->naxis1;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis1);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_NAXIS2:
			value = obj->naxis2;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis2);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_OBJECT:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->object);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->object;
		case FITS_OBSERVATORY:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->observatory);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->observatory;
		case FITS_OBSERVER:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->observer);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->observer;
		case FITS_POSTSCAN:
			value = obj->postscan;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->postscan);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_PRESCAN:
			value = obj->prescan;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->prescan);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)value;
		case FITS_RA:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->ra);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->ra;
		case FITS_SIDEREAL_TIME:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->sidereal_time);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->sidereal_time;
		case FITS_SIMPLE:
			char_value = obj->simple; 
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %c\n", modifier, obj->simple);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)char_value;
		case FITS_TELESCOPE:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->telescope);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->telescope;
		case FITS_UTSTART:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->ut_start);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->ut_start;

		case FITS_USER_KEYWORD1:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[0]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[0];
		case FITS_USER_KEYWORD2:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[1]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[1];
		case FITS_USER_KEYWORD3:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[2]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[2];
		case FITS_USER_KEYWORD4:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[3]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[3];
		case FITS_USER_KEYWORD5:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[4]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[4];
		case FITS_USER_KEYWORD6:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[5]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[5];
		case FITS_USER_KEYWORD7:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[6]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[6];
		case FITS_USER_KEYWORD8:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[7]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[7];
		case FITS_USER_KEYWORD9:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[8]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keywords[8];
		case FITS_USER_KEYWORD_VALUE1:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[0]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[0];
		case FITS_USER_KEYWORD_VALUE2:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[1]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[1];
		case FITS_USER_KEYWORD_VALUE3:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[2]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[2];
		case FITS_USER_KEYWORD_VALUE4:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[3]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[3];
		case FITS_USER_KEYWORD_VALUE5:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[4]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[4];
		case FITS_USER_KEYWORD_VALUE6:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[5]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[5];
		case FITS_USER_KEYWORD_VALUE7:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[6]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[6];
		case FITS_USER_KEYWORD_VALUE8:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[7]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[7];
		case FITS_USER_KEYWORD_VALUE9:
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[8]);
#endif	DEBUG_FITS_HEADER
			return (fits_opaque)obj->user_keyword_values[8];

		default:
			fprintf(stderr, 
				"\nlibfits.c:  fits_get() -- invalid tag %d",
				modifier);
	}

	return 0;
}


#ifdef	__STDC__
int fits_set(...)
#else	/* K&R C */
int
fits_set(va_alist)
va_dcl
#endif	__STDC__
{
	va_list		ap;
	FITS_OBJ	*obj;
	int		modifier, value, index=0;
	char		*buffer;
	char		char_value;
	double		double_value;
#ifdef	SBUSTOOL
	int		*ptr;
#else	/* VMETOOL */
	caddr_t		ptr;

#endif	/* SBUSTOOL */

	va_start(ap);
	obj = va_arg(ap, FITS_OBJ *);
	modifier = va_arg(ap, int);
	while (modifier != NULL)
	{
		switch (modifier)
		{
			case FITS_FD:
				value = va_arg(ap, int);
				obj->fd = value;
				break;
			case FITS_IMAGE_ADDR:
#ifndef	SBUSTOOL
				ptr = va_arg(ap, caddr_t);
				obj->image_addr = ptr;
#endif	/* SBUSTOOL */
				break;
			case FITS_DIRECTORY:
				buffer = va_arg(ap, char *);
				strncpy(obj->directory, buffer,
					strlen(buffer)+1);
				break;
			case FITS_FILENAME:
				buffer = va_arg(ap, char *);
				strncpy(obj->filename, buffer,
					strlen(buffer)+1);
				break;

			/* FITS Tags */
/* These tags are in alphabetical order to make it easier to find one. *\
 * The FITS_ and DEFAULT_FITS_ tags should be in the same order as     *
\* the fits_obj variables.                                             */
			case FITS_AIRMASS:
				obj->airmass = (float) va_arg(ap, double);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf\n", modifier, obj->airmass);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_BITPIX:
				value = va_arg(ap, int);
				obj->bitpix = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->bitpix);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_DATE_OBS:
				buffer = va_arg(ap, char *);
				strncpy(obj->date_observation, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->date_observation);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_DEC:
				buffer = va_arg(ap, char *);
				strncpy(obj->dec, buffer, strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->dec);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_DETECTOR:
				buffer = va_arg(ap, char *);
				strncpy(obj->detector, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->detector);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_EPOCH:
				obj->epoch = (float) va_arg(ap, double);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf\n", modifier, obj->epoch);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_EXPTIME:
				obj->exptime = (float) va_arg(ap, double);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %lf\n", modifier, obj->exptime);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_EXPTYPE:
				buffer = va_arg(ap, char *);
				strncpy(obj->exptype, buffer, strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->exptype);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_FILTER:
				buffer = va_arg(ap, char *);
				strncpy(obj->filter, buffer, strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->filter);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_INSTRUMENT:
				buffer = va_arg(ap, char *);
				strncpy(obj->instrument, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->instrument);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_IRAFNAME:
				buffer = va_arg(ap, char *);
				strncpy(obj->iraf_name, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->iraf_name);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_ISTART:
				value = va_arg(ap, int);
				obj->istart = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->istart);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_JSTART:
				value = va_arg(ap, int);
				obj->jstart = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->jstart);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_NAXIS:
				value = va_arg(ap, int);
				obj->naxis = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_NAXIS1:
				value = va_arg(ap, int);
				obj->naxis1 = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis1);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_NAXIS2:
				value = va_arg(ap, int);
			obj->naxis2 = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->naxis2);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_OBJECT:
				buffer = va_arg(ap, char *);
				strncpy(obj->object, buffer, strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->object);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_OBSERVATORY:
				buffer = va_arg(ap, char *);
				strncpy(obj->observatory, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->observatory);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_OBSERVER:
				buffer = va_arg(ap, char *);
				strncpy(obj->observer, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->observer);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_POSTSCAN:
				value = va_arg(ap, int);
				obj->postscan = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->postscan);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_PRESCAN:
				value = va_arg(ap, int);
				obj->prescan = value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %d\n", modifier, obj->prescan);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_RA:
				buffer = va_arg(ap, char *);
				strncpy(obj->ra, buffer, strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->ra);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_SIDEREAL_TIME:
				buffer = va_arg(ap, char *);
				strncpy(obj->sidereal_time, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->sidereal_time);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_SIMPLE:
				char_value = (char) va_arg(ap, char);
				obj->simple = char_value;
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\tvalue = %c\n", modifier, obj->simple);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_TELESCOPE:
				buffer = va_arg(ap, char *);
				strncpy(obj->telescope, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->telescope);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_UTSTART:
				buffer = va_arg(ap, char *);
				strncpy(obj->ut_start, buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s", modifier, obj->ut_start);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD1:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[0], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[0][index] != '\0';
					index++)
				{
					obj->user_keywords[0][index] =
					  toupper(obj->user_keywords[0][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[0]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD2:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[1], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[1][index] != '\0';
					index++)
				{
					obj->user_keywords[1][index] =
					  toupper(obj->user_keywords[1][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[1]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD3:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[2], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[2][index] != '\0';
					index++)
				{
					obj->user_keywords[2][index] =
					  toupper(obj->user_keywords[2][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[2]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD4:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[3], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[3][index] != '\0';
					index++)
				{
					obj->user_keywords[3][index] =
					  toupper(obj->user_keywords[3][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[3]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD5:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[4], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[4][index] != '\0';
					index++)
				{
					obj->user_keywords[4][index] =
					  toupper(obj->user_keywords[4][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[4]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD6:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[5], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[5][index] != '\0';
					index++)
				{
					obj->user_keywords[5][index] =
					  toupper(obj->user_keywords[5][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[5]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD7:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[6], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[6][index] != '\0';
					index++)
				{
					obj->user_keywords[6][index] =
					  toupper(obj->user_keywords[6][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[6]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD8:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[7], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[7][index] != '\0';
					index++)
				{
					obj->user_keywords[7][index] =
					  toupper(obj->user_keywords[7][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[7]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD9:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keywords[8], buffer,
					strlen(buffer)+1);
				for (index = 0;
					obj->user_keywords[8][index] != '\0';
					index++)
				{
					obj->user_keywords[8][index] =
					  toupper(obj->user_keywords[8][index]);
				}
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n", modifier, obj->user_keywords[8]);
#endif	DEBUG_FITS_HEADER
				break;

			case FITS_USER_KEYWORD_VALUE1:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[0], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[0]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE2:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[1], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[1]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE3:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[2], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[2]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE4:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[3], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[3]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE5:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[4], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[4]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE6:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[5], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[5]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE7:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[6], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[6]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE8:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[7], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[7]);
#endif	DEBUG_FITS_HEADER
				break;
			case FITS_USER_KEYWORD_VALUE9:
				buffer = va_arg(ap, char *);
				strncpy(obj->user_keyword_values[8], buffer,
					strlen(buffer)+1);
#ifdef	DEBUG_FITS_HEADER
printf("\nmodifier = %d\t\nvalue = %s\n",
	modifier, obj->user_keyword_values[8]);
#endif	DEBUG_FITS_HEADER
				break;

			default:
				fprintf(stderr,
				     "\nlibfits.c (fits_set) invalid tag %d\n",
				     modifier);
		}

		modifier = va_arg(ap, int);
	}
	return 0;
}


#ifdef	__STDC__
int fits_open(FITS_OBJ *obj)
#else	/* K&R C */
int
fits_open(obj)
FITS_OBJ	*obj;
#endif	__STDC__
{
	char	buffer[MAX_PATH_NAME];

	sprintf(buffer, "%s/%s", obj->directory, obj->filename);
	if ((obj->fd = open(buffer, O_RDWR | O_CREAT | O_TRUNC, 0666)) == -1) {
		fits_errno = FITS_OPEN_ERR;
		return -1;
	}
	return 1;
}


#ifdef	__STDC__
int fits_close(FITS_OBJ *obj)
#else	/* K&R C */
int
fits_close(obj)
FITS_OBJ	*obj;
#endif	__STDC__
{
	if ((close(obj->fd)) == -1) {
		fits_errno = FITS_CLOSE_ERR;
		return -1;
	}
	return 0;
}


#ifdef	__STDC__
int fits_write(FITS_OBJ *obj)
#else	/* K&R C */
int
fits_write(obj)
FITS_OBJ	*obj;
#endif	__STDC__
{
	int	num_bytes, total_size, i;
	char	*out_addr_base, *addr_ptr;

	num_bytes = obj->naxis1 * obj->naxis2 * (obj->bitpix / 8);
	total_size = FITS_HEADER_SIZE + num_bytes;

	lseek(obj->fd, total_size-1, SEEK_SET);
	write(obj->fd, "", 1);
	lseek(obj->fd, 0, SEEK_SET);

	put_fits_header(obj);

#ifndef	SBUSTOOL
	out_addr_base = mmap(0, total_size, PROT_READ | PROT_WRITE,
				MAP_SHARED, obj->fd, 0);

	addr_ptr = out_addr_base + FITS_HEADER_SIZE;

	for (i=0; i<num_bytes; i++)
		*(addr_ptr + i) = *(obj->image_addr+i);

	munmap(out_addr_base, total_size);
#endif	/* SBUSTOOL */
}


#ifdef	__STDC__
void put_fits_header(FITS_OBJ *obj)
#else	/* K&R C */
void
put_fits_header(obj)
FITS_OBJ	*obj;
#endif	__STDC__
{
	/* NOTE:  i is a counter that tracks the number of fits fields
	 *	    written to the fits header.  There is a 36 field
	 *	    limit.
	 */
	int i = 0, j;
	char card[81];

	strcpy(card, fits_format(obj, FITS_SIMPLE));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_BITPIX));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_NAXIS));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_NAXIS1));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_NAXIS2));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_PRESCAN));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_POSTSCAN));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_ISTART));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_JSTART));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_EXPTIME));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_EXPTYPE));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_IRAFNAME));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_OBSERVATORY));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_TELESCOPE));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_OBJECT));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_OBSERVER));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_DATE_OBS));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_UTSTART));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_SIDEREAL_TIME));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_RA));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_DEC));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_EPOCH));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_AIRMASS));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_INSTRUMENT));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_DETECTOR));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_FILTER));
	write_card(obj->fd, card); ++i;

	/* This section cycles through only those keywords that
	 * are initialized by the user.
	 */
	for (j = 0; j < MAX_USER_KEYWORDS; j++)
	{
#ifdef	DEBUG_FITS_HEADER
	printf("\n%d:  %s", j+FITS_USER_KEYWORD1, obj->user_keywords[j]);
#endif	/* DEBUG_FITS_HEADER */
		if (obj->user_keywords[j][0] != '\0')
		{
#ifdef	DEBUG_FITS_HEADER
	printf("\nIn if statement:  %d:  %s\n", j + FITS_USER_KEYWORD1,
		obj->user_keywords[j]);
#endif	/* DEBUG_FITS_HEADER */
			strcpy(card, fits_format(obj, j + FITS_USER_KEYWORD1));
			write_card(obj->fd, card); ++i;
		}
	}

	/* FITS_END and FITS_BLANK **must** be at the end */
	strcpy(card, fits_format(obj, FITS_END));
	write_card(obj->fd, card); ++i;

	strcpy(card, fits_format(obj, FITS_BLANK));

#ifdef DEBUG_FITS_HEADER
	if (i > 36)
		printf("\n***** ERROR *****:  fits_header overflow %d\n", i);
#endif DEBUG_FITS_HEADER
	while (i < 36)
	{
		write_card(obj->fd, card); ++i;
	}

#ifdef DEBUG_FITS_HEADER
	for( i = FITS_BEGIN_TAGS; i < FITS_END_TAGS; i++)
	{
	  strcpy(card, fits_format(obj, i));
	  printf("\n%s", card);
	}
	for (j = 0; j < MAX_USER_KEYWORDS; j++)
	{
		if (obj->user_keywords[j][0] != '\0')
			printf("%d:  %s\n", j, obj->user_keywords[j]);
			strcpy(card, fits_format(obj, j + FITS_USER_KEYWORD1));
	  		printf("\n%s", card);
	}

	printf("\nfits_format finished\n");
#endif DEBUG_FITS_HEADER
}

#ifdef	__STDC__
void write_card(int fd, char *str)
#else	/* K&R C */
void
write_card(fd, str)
int	fd;
char	*str;
#endif	__STDC__
{
	write(fd, str, 80);
}

#ifdef	__STDC__
void fits_perror(char *s)
#else	/* K&R C */
void
fits_perror(s)
char	*s;
#endif	__STDC__
{
	fprintf(stderr, "%s:  %s\n", s, fits_errlist[fits_errno]);
}

#ifdef	__STDC__
char *fits_format(FITS_OBJ *fits, int fits_item)
#else	/* K&R C */
char *fits_format(fits, fits_item)
FITS_OBJ *fits;
int fits_item;
#endif	__STDC__
{
  int i, j, index;

  char	temp[81],
	fits_label[10],
	fits_value[71];

  /* Determine FITS label */
  fits_label[0] = '\0';
  for (i = 0; i < (FITS_END_TAGS - FITS_BEGIN_TAGS); i++)
  {
     if (fios[i].tag == fits_item)
     {
       strcpy(fits_label, fios[i].string);
     }
  }

  if ((fits_label[0] == '\0') && (fits_item != FITS_BLANK))
  {
	strcpy(fits_label, "COMMENT  ");
  }

  /* format string */
  switch (fits_item)
  {
    /* Logicals */
    /* Integers (supports only the real part) */
    /* Floating Point Numbers (supports only the real part) */
    /* Character Strings */
    case FITS_AIRMASS:
	sprintf(temp, "%9s %20.2f%50s", fits_label,
		fits->airmass, " ");
    	break;

    case FITS_BITPIX:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->bitpix, " ");
    	break;

    case FITS_BLANK:
	sprintf(fits_value, "%s", " ");
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_COMMENT:
	sprintf(fits_value, "%s", "user comments");
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_DATE_OBS:
	sprintf(fits_value, "'%s'", fits->date_observation);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_DEC:
	sprintf(fits_value, "'%s'", fits->dec);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_DETECTOR:
	sprintf(fits_value, "'%s'", fits->detector);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_END:
	sprintf(fits_value, "%s", " ");
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_EPOCH:
	sprintf(temp, "%9s %20.1f%50s", fits_label,
		fits->epoch, " ");
    	break;

    case FITS_EXPTIME:
	sprintf(temp, "%9s %20.3f%50s", fits_label,
		fits->exptime, " ");
    	break;

    case FITS_EXPTYPE:
	sprintf(fits_value, "'%s'", fits->exptype);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_FILTER:
	sprintf(fits_value, "'%s'", fits->filter);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_HISTORY:
	sprintf(fits_value, "%s", "file history");
	sprintf(temp, "%9s %-70s", fits_label, "file history");
    	break;

    case FITS_INSTRUMENT:
	sprintf(fits_value, "'%s'", fits->instrument);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_IRAFNAME:
	sprintf(fits_value, "'%s'", fits->iraf_name);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_ISTART:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->istart, " ");
    	break;

    case FITS_JSTART:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->jstart, " ");
    	break;

    case FITS_NAXIS:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->naxis, " ");
    	break;

    case FITS_NAXIS1:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->naxis1, " ");
    	break;

    case FITS_NAXIS2:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->naxis2, " ");
    	break;

    case FITS_OBJECT:
	sprintf(fits_value, "'%s'", fits->object);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_OBSERVATORY:
	sprintf(fits_value, "'%s'", fits->observatory);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_OBSERVER:
	sprintf(fits_value, "'%s'", fits->observer);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_POSTSCAN:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->postscan, " ");
    	break;

    case FITS_PRESCAN:
	sprintf(temp, "%9s %20d%50s", fits_label,
		fits->prescan, " ");
    	break;

    case FITS_RA:
	sprintf(fits_value, "'%s'", fits->ra);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_SIDEREAL_TIME:
	sprintf(fits_value, "'%s'", fits->sidereal_time);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_SIMPLE:
	sprintf(temp, "%9s %20c%50s", fits_label,
		fits->simple, " ");
    	break;

    case FITS_TELESCOPE:
	sprintf(fits_value, "'%s'", fits->telescope);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_UTSTART:
	sprintf(fits_value, "'%s'", fits->ut_start);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
    	break;

    case FITS_USER_COMMENTS:
	sprintf(temp, "%9s %-70s", fits_label, "not implemented");
    	break;

    case FITS_USER_KEYWORD1:
	sprintf(fits_label, "%s", fits->user_keywords[0]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[0]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD2:
	sprintf(fits_label, "%s", fits->user_keywords[1]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[1]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD3:
	sprintf(fits_label, "%s", fits->user_keywords[2]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[2]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD4:
	sprintf(fits_label, "%s", fits->user_keywords[3]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[3]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD5:
	sprintf(fits_label, "%s", fits->user_keywords[4]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[4]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD6:
	sprintf(fits_label, "%s", fits->user_keywords[5]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[5]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD7:
	sprintf(fits_label, "%s", fits->user_keywords[6]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[6]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD8:
	sprintf(fits_label, "%s", fits->user_keywords[7]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[7]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    case FITS_USER_KEYWORD9:
	sprintf(fits_label, "%s", fits->user_keywords[8]);
	index = strlen(fits_label);
	while (index < 8)
	{
		fits_label[index] = ' ';
		fits_label[index+1] = '\0';
		index = strlen(fits_label);
	}

	/* Determine if a label is a comment or variable.
	 * Comment fields do not have '=' signs in the FITS header
	 * Variables do...
	 */
	if ((strncmp(fits_label, "COMMENT", 7) == 0) ||
	    (strncmp(fits_label, "HISTORY", 7) == 0))
	{
		fits_label[8] = ' ';
	}
	else
	{
		fits_label[8] = '=';
	}
	fits_label[9] = '\0';

	sprintf(fits_value, "'%s'", fits->user_keyword_values[8]);
	sprintf(temp, "%9s %-70s", fits_label, fits_value);
	break;

    /* Default Error */
    default:
	sprintf(temp, "\n***** ERROR ***** fits_format()\n");
	fprintf(stderr, "\n***** ERROR ***** fits_format()\n");
	fprintf(stderr, "Undefined fits_tag:  %d\n", fits_item);
	break;
  }

#ifdef DEBUG_FITS_HEADER
  printf("##### DEBUG_FITS_HEADER\t%d\t%s\n", fits_item, fits_label);
#endif DEBUG_FITS_HEADER

  return (temp);
}


/*
 * $Log: libfits.c,v $
 * Revision 1.14  1996/06/15 00:16:36  tjk
 * Added #include <time.h> for struct tm...
 *
 * Revision 1.13  1996/05/13  18:42:57  tjk
 * Changed the user keywords to allow for multiple COMMENT and HISTORY
 * entries.
 *
 * Revision 1.12  1996/05/13 16:48:01  tjk
 * Changed fits_create() to enter current date and time, as defaults.
 *
 * Revision 1.11  1996/05/07 17:20:01  tjk
 * Added user supplied keywords.
 *
 * Revision 1.10  1996/01/06 00:22:12  gumrukcu
 * fits_create has been changed.
 *
 * Revision 1.9  1996/01/02 23:14:23  gumrukcu
 * fits_set function should have remained same. Because fits_obj is forgotten to be initialized
 * in initialize.c. So it is returned to its very first version.
 *
 * Revision 1.8  1996/01/02  20:41:26  gumrukcu
 * The code for initialozation of fits_obj for SBUSTOOL is added.
 *
 * Revision 1.7  1995/12/12  22:33:23  tjk
 * Merged SBUS and VME versions
 *
 * Revision 1.6  1995/08/22  00:10:00  tjk
 * Corrected Log error from new RCS version.
 *
 * Revision 1.5  1995/08/21  23:46:21  tjk
 * Cleaning up code...
 *
 * Revision 1.4  1995/08/10  16:37:02  tjk
 * Enhanced FITS stuff.
 *
 * Revision 1.3  1995/06/13  23:54:23  tjk
 * Adding better FITS stuff...
 *
 * Revision 1.2  1995/02/22  20:40:16  tjk
 * Added a more complete FITS header.
 *
 * Revision 1.1  1995/02/10  03:39:27  tjk
 * Initial revision
 *
 *
 */
