/* $Header: /rosie/src/ccd_utilities/src/libmydb/RCS/libmydb.c,v 1.3 1995/12/12 22:33:25 tjk Exp $
 */

#include	<stdio.h>
#include	<ctype.h>
#include	<malloc.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<varargs.h>
#include	<unistd.h>
#include	<string.h>
#include	"mydb.h"

#define		FALSE		0
#define		TRUE		1
#define		OK		0
#define		NOT_OK		-1

#define		MYDB_MASTER	1000
#define		MYDB_LIMIT	1001

char	*mydb_errlist[] =
{
	"no error",
	"Cannot allocate memory for database object.",
	"No such database file.",
};

typedef struct mydb_record {
	struct mydb_record	*master_next;
	struct mydb_record	*limit_next;
	char			*attribute;
	char			*value;
	int			type;
	int			dirty;
} MYDB_RECORD;
	
typedef	struct _mydb_obj {
	char		*filename;
	char		*resource_type;
	int		total_records;
	int		limit_records;
	int		bad_records;
	int		dirty;
	int		append;
	MYDB_RECORD	*master_ptr;
	MYDB_RECORD	*limit_ptr;
} MYDB_OBJ;


/*>>>>>  Function Prototypes  <<<<<*/
#ifdef	__STDC__
mydb_opaque *mydb_create(void);
int mydb_open(MYDB_OBJ *obj, char *filename);
int mydb_free(MYDB_OBJ *obj);
int mydb_destroy(MYDB_OBJ *obj);
static int mydb_read_db(MYDB_OBJ *obj, FILE *fp);
static int mydb_extract_resource(MYDB_OBJ *obj);
mydb_opaque mydb_get(...);
int mydb_set(...);
int mydb_delete(...);
int mydb_write(...);
static int write_db(MYDB_OBJ *obj, char *filename, char *resource);
static MYDB_RECORD *search(MYDB_OBJ *obj, char *attribute, int list);
static int match(char *s1, char *s2, char *terminator);
static int mydb_strpos(char *buffer, char c);
static char *lowercase(char *buffer);
static int
    mydb_insert_record(MYDB_OBJ *obj, char *attribute, char *value, int dirty);
int mydb_print_records(MYDB_OBJ *obj);
mydb_opaque get_typed_value(int type, char *char_value);

#else	/* K&R C */
mydb_opaque *mydb_create();
int mydb_open();
int mydb_free();
int mydb_destroy();
static int mydb_read_db();
static int mydb_extract_resource();
mydb_opaque mydb_get();
int mydb_set();
int mydb_delete();
int mydb_write();
static int write_db();
static MYDB_RECORD *search();
static int match();
static int mydb_strpos();
static char *lowercase();
static int
    mydb_insert_record();
int mydb_print_records();
mydb_opaque get_typed_value();
#endif	/* __STDC__ */
/*>>>>>  End Function Prototypes  <<<<<*/


#ifdef	__STDC__
mydb_opaque *mydb_create(void)
#else	/* K&R C */
mydb_opaque
*mydb_create()
#endif	/* __STDC__ */
{
	MYDB_OBJ	*obj;

	if ((obj = (MYDB_OBJ *)malloc(sizeof(MYDB_OBJ))) != NULL)
	{
		obj->filename = NULL;
		obj->resource_type = NULL;
		obj->total_records = 0;
		obj->limit_records = 0;
		obj->bad_records = 0;
		obj->append = FALSE;
		obj->master_ptr = NULL;
		obj->limit_ptr = NULL;
	}
	else
	{
		mydb_errno = MYDB_CREATE_ERR;
	}

	return (mydb_opaque *)obj;
}


#ifdef	__STDC__
int mydb_open(MYDB_OBJ *obj, char *filename)
#else	/* K&R C */
int
mydb_open(obj, filename)
MYDB_OBJ	*obj;
char		*filename;
#endif	/* __STDC__ */
{
	FILE		*fp;

	obj->filename = strdup(filename);
	if ((fp=fopen(obj->filename, "r")) == NULL)
	{
		mydb_errno = MYDB_OPEN_ERR;
		free(obj->filename);
		obj->filename = NULL;
		return -1;
	}

	mydb_read_db(obj, fp);
	fclose(fp);
	return 1;
}


#ifdef	__STDC__
int mydb_free(MYDB_OBJ *obj)
#else	/* K&R C */
int
mydb_free(obj)
MYDB_OBJ	*obj;
#endif	/* __STDC__ */
{
	MYDB_RECORD	*first, *ptr;

	if (obj != NULL)
	{
		first = obj->master_ptr;
		if (first)
		{
			ptr = first->master_next;
			while (ptr)
			{
				free(first->attribute);
				free(first->value);
				free(first);
				first = ptr;
				ptr = ptr->master_next;
			}
			free(first->attribute);
			free(first->value);
			free(first);
		}
		if (obj->filename)
			free(obj->filename);
		obj->filename = NULL;
		if (obj->resource_type)
			free(obj->resource_type);
		obj->resource_type = NULL;
		obj->total_records = 0;
		obj->limit_records = 0;
		obj->bad_records = 0;
		obj->dirty = FALSE;
		obj->master_ptr = NULL;
		obj->limit_ptr = NULL;
	}
}


#ifdef	__STDC__
int mydb_destroy(MYDB_OBJ *obj)
#else	/* K&R C */
int
mydb_destroy(obj)
MYDB_OBJ	*obj;
#endif	/* __STDC__ */
{
	if (obj != NULL)
	{
		mydb_free(obj);
		free (obj);
		obj = NULL;
	}
}


#ifdef	__STDC__
static int mydb_read_db(MYDB_OBJ *obj, FILE *fp)
#else	/* K&R C */
static int
mydb_read_db(obj, fp)
MYDB_OBJ	*obj;
FILE		*fp;
#endif	/* __STDC__ */
{
	char	buffer[MYDB_MAXBUFLEN];
	char	tmpbuffer[MYDB_MAXBUFLEN];
	char	*attribute, *value;
	int	colon;

	while (fgets(buffer, MYDB_MAXBUFLEN, fp) != NULL)
	{
		buffer[strlen(buffer)-1] = '\0';

		if (strlen(buffer) == 0)
		{
			continue; 
		}
		if ((colon = mydb_strpos(buffer, ':')) != -1)
		{
			strcpy(tmpbuffer, buffer);
			attribute = strtok(tmpbuffer, ":");
		}
		value = &buffer[colon];
		if (*value == '\0')
		{
			obj->bad_records++;
			continue;
		}
		else
		{
			while (isspace(*(++value)));
		}
		if (strlen(attribute) && strlen(value))
		{
			mydb_insert_record(obj, attribute, value, FALSE);
		}
		else
		{
			obj->bad_records++;
		}
	}
}


#ifdef	__STDC__
static int mydb_extract_resource(MYDB_OBJ *obj)
#else	/* K&R C */
static int
mydb_extract_resource(obj)
MYDB_OBJ	*obj;
#endif	/* __STDC__ */
{
	MYDB_RECORD	*master_ptr;
	int		len;

	master_ptr = obj->master_ptr;
	obj->limit_records = 0;
	obj->limit_ptr = NULL;
	len = strlen(obj->resource_type);
	while(master_ptr) {
		if ((!len) ||
		    (match(master_ptr->attribute, obj->resource_type, "")))
		{
			master_ptr->limit_next = obj->limit_ptr;
			obj->limit_ptr = master_ptr;
			obj->limit_records++;
		}
		master_ptr = master_ptr->master_next;
	}
}


#ifdef	__STDC__
mydb_opaque mydb_get(...)
#else	/* K&R C */
mydb_opaque
mydb_get(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list		ap;
	MYDB_OBJ	*obj;
	int		modifier;
	char		*attribute;
	MYDB_RECORD	*record;

	va_start(ap);
	obj = va_arg(ap, MYDB_OBJ *);
	modifier = va_arg(ap, int);

	switch (modifier)
	{
		case MYDB_ATTRIBUTE:
			attribute = va_arg(ap, char *);
			record = search(obj, attribute, MYDB_LIMIT);
			if (record != NULL)
				return (mydb_opaque)record->value;
			else 
				return (mydb_opaque)NULL;
		case MYDB_FILENAME:
			return (mydb_opaque)obj->filename;
		case MYDB_RESOURCE_TYPE:
			return (mydb_opaque)obj->resource_type;
		case MYDB_TOTAL_RECORDS:
			return (mydb_opaque)obj->total_records;
		case MYDB_LIMIT_RECORDS:
			return (mydb_opaque)obj->limit_records;
		case MYDB_BAD_RECORDS:
			return (mydb_opaque)obj->bad_records;
	}
}


#ifdef	__STDC__
int mydb_set(...)
#else	/* K&R C */
int
mydb_set(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list		ap;
	MYDB_OBJ	*obj;
	MYDB_RECORD	*record;
	int		modifier;
	char		*attribute, *resource;
	char		*value;
	char		buffer[255];

	va_start(ap);
	obj = va_arg(ap, MYDB_OBJ *);
	modifier = va_arg(ap, int);

	switch (modifier)
	{
		case MYDB_RESOURCE_TYPE:
			resource = va_arg(ap, char *);
			if (obj->resource_type != NULL)
				free(obj->resource_type);
			obj->resource_type = strdup(resource);
			mydb_extract_resource(obj);
			return OK;
		case MYDB_FILENAME:
			value = va_arg(ap, char *);
			if (obj->filename != NULL)
				free(obj->filename);
			obj->filename = strdup(value);
			return OK;
		case MYDB_APPEND:
			obj->append = va_arg(ap, int);
			return OK;
		case MYDB_ATTRIBUTE:
			attribute = va_arg(ap, char *);
			value = va_arg(ap, char *);
			record = search(obj, attribute, MYDB_LIMIT);
			if (record == NULL)
			{
				if (strlen(obj->resource_type))
				{
					sprintf(buffer, "%s.%s",
						obj->resource_type, attribute);
				}
				else
				{
					sprintf(buffer, "%s", attribute);
				}
				mydb_insert_record(obj, buffer, value, TRUE);
				record = obj->master_ptr;
				record->limit_next = obj->limit_ptr;
				obj->limit_ptr = record;
				obj->dirty = TRUE;
				return(OK);
			}
			else
			{
				free(record->value);
				record->value = strdup(value);
				record->dirty = TRUE;
				obj->dirty = TRUE;
				return(OK);
			}
	}
}


#ifdef	__STDC__
int mydb_delete(...)
#else	/* K&R C */
int
mydb_delete(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list		ap;
	MYDB_OBJ	*obj;
	MYDB_RECORD	*record;
	int		modifier;
	char		*attribute, *resource;

	va_start(ap);
	obj = va_arg(ap, MYDB_OBJ *);
	modifier = va_arg(ap, int);

	while(modifier != NULL)
	{
		switch (modifier)
		{
			case MYDB_RESOURCE_TYPE:
				resource = va_arg(ap, char *);
				break;
			case MYDB_ATTRIBUTE:
				attribute = va_arg(ap, char *);
				break;
			default:
				printf("delete error\n");
		}
		modifier = va_arg(ap, int);
	}
}


#ifdef	__STDC__
int mydb_write(...)
#else	/* K&R C */
int
mydb_write(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list		ap;
	MYDB_OBJ	*obj;
	MYDB_RECORD	*record;
	int		modifier;
	char		*filename;
	char		*attribute, *value, *resource;

	va_start(ap);
	obj = va_arg(ap, MYDB_OBJ *);
	modifier = va_arg(ap, int);
	resource = obj->resource_type;
	filename = obj->filename;

	while(modifier != NULL)
	{
		switch (modifier)
		{
			case MYDB_FILENAME:
				filename = va_arg(ap, char *);
				break;
			case MYDB_RESOURCE_TYPE:
				resource = va_arg(ap, char *);
				break;
			default:
				printf("write parameter error\n");
		}
		modifier = va_arg(ap, int);
	}
	write_db(obj, filename, resource);
}


#ifdef	__STDC__
static int write_db(MYDB_OBJ *obj, char *filename, char *resource)
#else	/* K&R C */
static int
write_db(obj, filename, resource)
MYDB_OBJ	*obj;
char		*filename;
char		*resource;
#endif	/* __STDC__ */
{
	FILE		*fp;
	MYDB_RECORD	*ptr;
	int		len;

	len = strlen(resource);
	if (obj->append)
	{
		if ((fp=fopen(filename, "a")) == NULL)
		{
			mydb_errno = MYDB_OPEN_ERR;
			return NOT_OK;
		}
	}
	else
	{
		if ((fp=fopen(filename, "w")) == NULL)
		{
			mydb_errno = MYDB_OPEN_ERR;
			return NOT_OK;
		}
	}

	ptr = obj->master_ptr;

	while (ptr != NULL)
	{
		if ((!len) || match(ptr->attribute, resource, "."))
		{
			fprintf(fp, "%s:\t%s\n", ptr->attribute, ptr->value);
		}
		ptr = ptr->master_next;
	}
	fclose(fp);
	return OK;
}


#ifdef	__STDC__
static MYDB_RECORD *search(MYDB_OBJ *obj, char *attribute, int list)
#else	/* K&R C */
static MYDB_RECORD 
*search(obj, attribute, list)
MYDB_OBJ	*obj;
char		*attribute;
int		list;
#endif	/* __STDC__ */
{

	char		buffer[MYDB_MAXBUFLEN];
	MYDB_RECORD	*ptr;

	if (list == MYDB_MASTER)
	{
		ptr = obj->master_ptr;
		sprintf(buffer, "%s",  attribute);
	}
	else if ((list == MYDB_LIMIT) && (strlen(obj->resource_type) == 0))
	{
		ptr = obj->limit_ptr;
		sprintf(buffer, "%s",  attribute);
	}
	else
	{
		ptr = obj->limit_ptr;
		sprintf(buffer, "%s.%s", obj->resource_type, attribute);
	}
	while(ptr != NULL)
	{
		if (match(ptr->attribute, buffer, "") == 1)
		{
			return(ptr);
		}
		else
		{
			switch (list)
			{
				case MYDB_MASTER:
					ptr = ptr->master_next;
					break;
				case MYDB_LIMIT:
					ptr = ptr->limit_next;
					break;
			}
		}
	}
	return (MYDB_RECORD *)NULL;
}


#ifdef	__STDC__
static int match(char *s1, char *s2, char *terminator)
#else	/* K&R C */
static int
match(s1, s2, terminator)
char	*s1, *s2;
char	*terminator;
#endif	/* __STDC__ */
{
	char	c1, c2;

	while (*s2)
	{
		c1 = tolower(*s1);
		c2 = tolower(*s2);
		if (c1 != c2)
			return FALSE;
		s1++;
		s2++;
	}

	if (*terminator == '\0')
		return TRUE;
	while (*terminator)
	{
		if (*s1 == *terminator) 
			return TRUE;
		else 
			terminator++;
	}
	return FALSE;
}


#ifdef	__STDC__
static int mydb_strpos(char *buffer, char c)
#else	/* K&R C */
static int
mydb_strpos(buffer, c)
char	*buffer;
char	c;
#endif	/* __STDC__ */
{
	char	*ptr;
	int	pos;

	pos = 0;
	ptr = buffer;
	while (*ptr != '\0')
	{
		if (*ptr == c)
		{
			return pos;
		}
		else
		{
			ptr++;
			pos++;
		}
	}
	return -1;
}


#ifdef	__STDC__
static char *lowercase(char *buffer)
#else	/* K&R C */
static char
*lowercase(buffer)
char	*buffer;
#endif	/* __STDC__ */
{
	char	*lbuffer;
	char	*s;

	lbuffer = strdup(buffer);
	s = lbuffer;
	while (*s)
	{
		*s = tolower(*s);
		s++;
	}
	return lbuffer;
}


#ifdef	__STDC__
static int
    mydb_insert_record(MYDB_OBJ *obj, char *attribute, char *value, int dirty)
#else	/* K&R C */
static int
mydb_insert_record(obj, attribute, value, dirty)
MYDB_OBJ	*obj;
char		*attribute;
char		*value;
int		dirty;
#endif	/* __STDC__ */
{
	MYDB_RECORD	*record;

	record = (MYDB_RECORD *)malloc(sizeof(MYDB_RECORD));
	record->attribute = strdup(attribute);
	record->value = strdup(value);
	record->dirty = dirty;
	record->master_next = obj->master_ptr;
	obj->master_ptr = record;
	obj->total_records++;
}


#ifdef	__STDC__
int mydb_print_records(MYDB_OBJ *obj)
#else	/* K&R C */
int
mydb_print_records(obj)
MYDB_OBJ	*obj;
#endif	/* __STDC__ */
{
	MYDB_RECORD	*ptr;

	printf("\nfilename: %s\nresource_type: %s\ntotal_records: %d\nbad_records: %d\nlimit_records: %d\n\n", obj->filename, obj->resource_type, obj->total_records, obj->bad_records, obj->limit_records);
	ptr = obj->limit_ptr;
	while (ptr != NULL)
	{
		printf("%s  %s\n", ptr->attribute, ptr->value);
		ptr = ptr->limit_next;
	}
}


#ifdef	__STDC__
mydb_opaque get_typed_value(int type, char *char_value)
#else	/* K&R C */
mydb_opaque
get_typed_value(type, char_value)
int	type;
char	*char_value;
#endif	/* __STDC__ */
{

	switch	(type)
	{
		case MYDB_BOOLEAN:
			return TRUE;
			break;
		case MYDB_BINARY:
			break;
		case MYDB_OCTAL:
			break;
		case MYDB_HEX:
			break;
		case MYDB_INTEGER:
			break;
		case MYDB_FLOAT:
			break;
		case MYDB_CHAR:
			break;
		default:
			printf("ERROR\n");
	}
}


/*
 * $Log: libmydb.c,v $
 * Revision 1.3  1995/12/12 22:33:25  tjk
 * Merged SBUS and VME versions
 *
 * Revision 1.2  1995/08/22  17:36:03  tjk
 * ANSI conversion
 *
 * Revision 1.1  1995/02/10  03:47:37  tjk
 * Initial revision
 *
 *
 */
