/* $Header: /rosie/src/ccd_utilities/tool/RCS/expctl.c,v 1.46 1997/01/03 22:38:37 gumrukcu Exp $
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/notice.h>
#include "ccd_conf.h"
#include "ccd.h"
#include "expctl.h"
#include "expctl_error.h"
#include "message.h"
#include "temperature.h"
#include "sao-iis.h"

#define TOM_TIMER 0


		/*>>>>>  Function Prototypes <<<<<*/

int	do_expose(void);
int	check_expose(void);
int	show_expctl_error(int errno);
int	do_readout_ccd(expctl_opaque *expctl, setup_opaque *setup,
			dsp_opaque *dsp, fits_opaque *fits);
int	do_save(expctl_opaque *expctl, setup_opaque *setup, fits_opaque *fits);
int	do_display(fits_opaque *fits);
int	increment_filename(expctl_opaque expctl);
int	expose_busy(int value);
int	read_busy(int value);
int     save_busy(int value);
int	abort_expose(void);
int	abort_read_save();
int 	pause_expose();
void	countdown_update(void);
int	save_to_disk(void);
int	read_out_ccd(void);

		/*>>>>>  End Function Prototypes <<<<<*/


struct itimerval	timer;
int	countdown=0;
int	target_time = 0;
int	check_with_dsp = 0;
int 	beeper = FALSE;
int     dual_image_mode=1;


#ifdef	__STDC__
int do_expose(void)
#else	/* K&R C */
int
do_expose()
#endif	/* __STDC__ */
{
	expctl_opaque	*expctl;
	setup_opaque	*setup;
	dsp_opaque	*dsp;
	fits_opaque	*fits;
	int		errno;
	int 		dly;
	char 		buffer[255];

	expctl = ccd_obj->expctl_obj;
	setup = ccd_obj->setup_obj;
	dsp = ccd_obj->dsp_obj;
	fits = ccd_obj->fits_obj;

	store_expctl_settings(expctl);

	send_reset(dsp);

	if ((errno = check_expose()) != 1) {
		show_expctl_error(errno);
	}
	else {
		message_write("**************************\n");
		message_write("Starting exposure sequence...\n");
		expose_busy(TRUE);


		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
		    DO_DUAL_IMAGE) && !(int)dsp_get(dsp, DSP_ABORT))
		{
			dual_image_mode=0;
		}
		else
		{
			dual_image_mode=1;
		}
	
		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
                    DO_BEEP) && !(int)dsp_get(dsp, DSP_ABORT))
		{
                       	beeper = TRUE; 
		}
		else
		{
			beeper = FALSE;
		}
		
		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
                    DO_SAVE_IMAGE) && !(int)dsp_get(dsp, DSP_ABORT))
                {
                        do_save(expctl, setup, fits);
                }

		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
		    DO_READOUT_CCD) && !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_readout_ccd(expctl, setup, dsp, fits);
		}


		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
		    DO_DISPLAY_IMAGE) &&(int)get_expctl_value((EXPCTL_OBJ *)expctl,
                    DO_READOUT_CCD) && !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_display(fits);
		}
                else 
                  if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
                      DO_DISPLAY_IMAGE) && !(int)get_expctl_value((EXPCTL_OBJ *)expctl,
                      DO_READOUT_CCD) && !(int)dsp_get(dsp, DSP_ABORT))
                  {
                       message_popup(Expctl->expctl_popup,
                                MESSAGE, "You must READOUT_CCD",
                                MESSAGE, "in order to display an image.",
                       NULL);
                  }

		if ((int) dsp_get(dsp, DSP_ABORT))
			message_write("Exposure Aborted.\n");
		else
			message_write("Exposure sequence complete.\n");
		readout_busy(TRUE);
                expose_busy(FALSE);
		write_frame(Expctl->expctl_popup, FRAME_LEFT_FOOTER, "");
		sprintf(buffer,"%10.3f",target_time/1000.0);
		xv_set(Expctl->exposure_time_setting, PANEL_VALUE,buffer, NULL);
	}


	ccd_obj->next_func_ptr = NULL;
}

 
#ifdef  __STDC__
int change_time(dsp_opaque *dsp,int exp_time)
#else   /* K&R C */
int
change_time(dsp, exp_time)
int             exp_time;
dsp_opaque      *dsp;
#endif  /* __STDC__ */
{
        int             new_time;
        char            buffer[255];
 
        strcpy(buffer, (char *)xv_get(Expctl->change_time_field,PANEL_VALUE));
        sscanf(buffer,"%d",&new_time);
 
        new_time = (new_time*1000);
	target_time = new_time;
        dsp_command((DSP_ATTR *)dsp, TIM_ID, WRM,Y, 24, new_time);
        new_time -= exp_time;
 
        return(new_time);
 
}


#ifdef	__STDC__
int check_expose(void)
#else	/* K&R C */
int
check_expose()
#endif	/* __STDC__ */
{

	expctl_opaque	*expctl;
	setup_opaque	*setup;
	char		*filename, *directory;
	char		buffer[255];
	int		exp_time;
	int		fd;

	expctl = ccd_obj->expctl_obj;
	setup  = ccd_obj->setup_obj;

	exp_time = (int)get_expctl_value((EXPCTL_OBJ *)expctl, EXPOSURE_TIME);
	filename = (char *)get_expctl_value((EXPCTL_OBJ *)expctl,
			IMAGE_FILENAME);
	directory = (char *)get_expctl_value((EXPCTL_OBJ *)expctl,
			IMAGE_DIRECTORY);

	if (!(int)get_setup_value(ccd_obj->setup_obj, CCD_SETUP_COMPLETE))
	{
		return CCD_SETUP_NOT_COMPLETE;
	}

	if ((int)get_expctl_value((EXPCTL_OBJ *)expctl, DO_READOUT_CCD))
	{
		if (exp_time == UNDEFINED)
			return EXPOSURE_TIME_UNDEFINED;
	}
	if ((int)get_expctl_value((EXPCTL_OBJ *)expctl, DO_SAVE_IMAGE))
	{
		if ((filename == NULL) || (strlen(filename) == 0))
			return IMAGE_FILENAME_UNDEFINED;
		sprintf(buffer, "%s/%s", directory, filename);
		if ((fd = open(buffer, O_RDWR | O_CREAT, 0666)) == -1)
		{
			return IMAGE_UNWRITABLE;
		}
		else
		{
			close(fd);
		}
	}
	if (!(int)get_setup_value(setup,DO_DIMENSIONS)) 
                {
			return DIM_UNDEFINED;
		}


	return 1;
}


#ifdef	__STDC__
int show_expctl_error(int errno)
#else	/* K&R C */
int
show_expctl_error(errno)
int	errno;
#endif	/* __STDC__ */
{
	switch (errno)
	{
		case EXPOSURE_TIME_UNDEFINED:
			message_popup(Expctl->expctl_popup,
				MESSAGE, "You must specify an",
				MESSAGE, "exposure time.",
			NULL);
			break;
		case IMAGE_FILENAME_UNDEFINED:
			message_popup(Expctl->expctl_popup,
				MESSAGE, "You must specify a filename",
				MESSAGE, "to save the image.",
			NULL);
			break;
		case CCD_SETUP_NOT_COMPLETE:
			message_popup(Expctl->expctl_popup,
				MESSAGE, "The IR Array interface has not been",
				MESSAGE, "configured.  Please select",
				MESSAGE, "'Configure IR Array...' from the",
				MESSAGE, " Setup menu.",
			NULL);
			break;
		case IMAGE_UNWRITABLE:
			message_popup(Expctl->expctl_popup,
				MESSAGE, "The image file specified",
				MESSAGE, "cannot be created.",
			NULL);
			break;
		case DIM_UNDEFINED:
                        message_popup(Setup->setup_popup,
                                MESSAGE,"You have to specify dimensions",
                                MESSAGE,"in order to complete setup",
                        NULL);
		default:
			printf("ERROR\n");
			break;	
	}
}


#ifdef  __STDC__
int save_to_disk(void)
#else   /* K&R C */
int
save_to_disk()
#endif  /* __STDC__ */
{
        int     ncols, nrows, nccds, msecs;
#ifdef	SBUSTOOL
        struct  itimerval       timer;
        int     naxis1, naxis2, bitpix, numbytes, i;
        char    *directory, *filename;
        int     imagesave;
#endif /* SBUSTOOL */

 	expctl_opaque   *expctl;
        setup_opaque    *setup;
        dsp_opaque      *dsp;
        fits_opaque     *fits;
 
        save_busy(TRUE);
	
        expctl = ccd_obj->expctl_obj;
        setup = ccd_obj->setup_obj;
        dsp = ccd_obj->dsp_obj;
        fits = ccd_obj->fits_obj;
#ifdef  SBUSTOOL
        store_expctl_settings(expctl);
	do_save(expctl, setup, fits); 
        message_write("\nSaving the image to disk...\n");
        directory = (char *)get_expctl_value((EXPCTL_OBJ *)expctl,
		IMAGE_DIRECTORY);
        filename = (char *)get_expctl_value((EXPCTL_OBJ *)expctl,
		IMAGE_FILENAME);
        imagesave = (int)get_expctl_value((EXPCTL_OBJ *)expctl, DO_SAVE_IMAGE);
        ncols = (int)get_setup_value(setup, NCOLS);
        nrows = (int)get_setup_value(setup, NROWS);
        nccds = 1; /* (int)get_setup_value(setup, NCCDS); */
        if(nccds == 4)
           fits_set(fits, FITS_NAXIS1, ncols*2, FITS_NAXIS2,nrows*2,NULL);
        else
           fits_set(fits, FITS_NAXIS1, ncols, FITS_NAXIS2,nrows*nccds,NULL);
        msecs = (int)get_expctl_value((EXPCTL_OBJ *)expctl, EXPOSURE_TIME);
        naxis1 = (fits_opaque)fits_get(fits, FITS_NAXIS1);
        naxis2 = (fits_opaque)fits_get(fits, FITS_NAXIS2);
        bitpix = (fits_opaque)fits_get(fits, FITS_BITPIX);
        numbytes = naxis1 * naxis2 * (bitpix/8);
        save_image( directory, filename, numbytes);
	message_write("Save complete.\n");
#else  /* VMETOOL */
	do_save(expctl, setup, fits);
/*
	fits_open((FITS_OBJ *)fits);
        if ((fits_write((FITS_OBJ *)fits)) == -1)
        {
                message_write(Expctl->expctl_popup,
                        MESSAGE, "Cannot save image to",
                        MESSAGE, "specified file.",
                NULL);
                message_write("Save ABORTED.\n");
        }
        else
        {
                if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
                    DO_AUTO_INCREMENT))
                {
                        increment_filename((expctl_opaque)expctl);
                }
                message_write("Save complete.\n");
        }
 
        fits_close((FITS_OBJ *)fits);
*/
#endif  /* SBUSTOOL */

	save_busy(FALSE);
	ccd_obj->next_func_ptr = NULL;
}
#ifdef  __STDC__
int read_out_ccd(void)
#else   /* K&R C */
int
read_out_ccd()
#endif  /* __STDC__ */
{
	int     ncols, nrows, nccds, msecs;
	expctl_opaque   *expctl;
        setup_opaque    *setup;
        dsp_opaque      *dsp;
        fits_opaque     *fits;
#ifdef  SBUSTOOL
        int     naxis1, naxis2, bitpix, numbytes, i;
        char    *directory, *filename;
        int     imagesave;
#else   /* VMETOOL */
	int  vme_image_addr;
#endif  /* SBUSTOOL */ 
#ifdef	IRTOOL
	message_write("\nReading out NICMOS3 Array...\n");
#else	/* CCDTOOL */
	message_write("\nReading out IR Array...\n");
#endif	/* IRTOOL */
	read_busy(TRUE); 
        expctl = ccd_obj->expctl_obj;
        setup = ccd_obj->setup_obj;
        dsp = ccd_obj->dsp_obj;
        fits = ccd_obj->fits_obj;
#ifdef  SBUSTOOL
	/*do_save(expctl, setup, fits); */
        store_expctl_settings(expctl);
 
        directory = (char *)get_expctl_value((EXPCTL_OBJ *)expctl, IMAGE_DIRECTORY);
        filename = (char *)get_expctl_value((EXPCTL_OBJ *)expctl, IMAGE_FILENAME);
        imagesave = (int)get_expctl_value((EXPCTL_OBJ *)expctl, DO_SAVE_IMAGE);
        ncols = (int)get_setup_value(setup, NCOLS);
        nrows = (int)get_setup_value(setup, NROWS);
        nccds = 1; /* (int)get_setup_value(setup, NCCDS); */
        if(nccds == 4)
           fits_set(fits, FITS_NAXIS1, ncols*2, FITS_NAXIS2,nrows*2,NULL);
        else
           fits_set(fits, FITS_NAXIS1, ncols, FITS_NAXIS2,nrows*nccds,NULL);
        msecs = (int)get_expctl_value((EXPCTL_OBJ *)expctl, EXPOSURE_TIME);
        naxis1 = (fits_opaque)fits_get(fits, FITS_NAXIS1);
        naxis2 = (fits_opaque)fits_get(fits, FITS_NAXIS2);
        bitpix = (fits_opaque)fits_get(fits, FITS_BITPIX);
        numbytes = naxis1 * naxis2 * (bitpix/8);
        dsp_command((DSP_ATTR *)dsp,TIM_ID, RDC, directory, filename, -1, numbytes, msecs);
	message_write("Read out complete...\n");
#else   /* VMETOOL */

	vme_image_addr = (int)dsp_get(dsp, VME_DATA_START);

	dsp_command((DSP_ATTR *)dsp, VME_ID, WRM, Y, 65, vme_image_addr & LOW_MASK);
	dsp_command((DSP_ATTR *)dsp, VME_ID, WRM, Y, 64, (vme_image_addr &
						(unsigned) HIGH_MASK)>>16);
	 dsp_command((DSP_ATTR *)dsp,TIM_ID,RDC);
/*
	if (countdown >= MIN_TIMER_VALUE)
	{
		timer.it_value.tv_sec = 1;
		timer.it_interval.tv_sec = 1;
		notify_set_itimer_func(Expctl->expctl_popup,
			(Notify_func) countdown_update, ITIMER_REAL,
			&timer, NULL);
		dsp_command((DSP_ATTR *)dsp, TIM_ID, SEX);
		notify_set_itimer_func(Expctl->expctl_popup,
			NOTIFY_FUNC_NULL, ITIMER_REAL, NULL, NULL);
	}
	else
	{
		dsp_command((DSP_ATTR *)dsp, TIM_ID, SEX);
	}
*/
#endif	/* SBUSTOOL */
	read_busy(FALSE);
        ccd_obj->next_func_ptr = NULL;
}


#ifdef	__STDC__
int do_readout_ccd(expctl_opaque *expctl, setup_opaque *setup,
		dsp_opaque *dsp, fits_opaque *fits)
#else	/* K&R C */
int
do_readout_ccd(expctl, setup, dsp, fits)
expctl_opaque	*expctl;
setup_opaque	*setup;
dsp_opaque	*dsp;
fits_opaque	*fits;
#endif	/* __STDC__ */
{
	int	ncols, nrows, nccds, msecs;
	int	naxis1, naxis2, bitpix, numbytes, i;
	char	*directory, *filename;
	char 	buffer[255];
	int	imagesave;
	int	vme_image_addr;

	ncols = (int)get_setup_value(setup, NCOLS);
	nrows = (int)get_setup_value(setup, NROWS);
	nccds = 1;/* (int)get_setup_value(setup, NCCDS); */
	target_time = msecs = (int)get_expctl_value((EXPCTL_OBJ *)expctl,
				EXPOSURE_TIME);

	dsp_command((DSP_ATTR *)dsp, TIM_ID, WRM, X, 1, msecs);

	countdown = msecs / 1000;
	
	directory = (char *) get_expctl_value((EXPCTL_OBJ *)expctl, IMAGE_DIRECTORY);
	filename = (char *) get_expctl_value((EXPCTL_OBJ *)expctl, IMAGE_FILENAME);
	imagesave = (int) get_expctl_value((EXPCTL_OBJ *)expctl, DO_SAVE_IMAGE);
	naxis1 = (fits_opaque) fits_get(fits, FITS_NAXIS1);
	naxis2 = (fits_opaque) fits_get(fits, FITS_NAXIS2);
	bitpix = (fits_opaque) fits_get(fits, FITS_BITPIX);
	numbytes = naxis1 * naxis2 * (bitpix / 8);

	 sprintf(buffer,"%10.3f",msecs/1000.0);
         xv_set(Expctl->change_time_field, PANEL_VALUE,buffer, NULL);

	/* NOTE:  The timer stuff is commented out in the sbus version.
	 *        However, I have left it in to see if it needed to be
	 *        commented out.  Also, the sbus SEX command contains
	 *        more parameters than the vme version.
	 */

	send_reset(dsp);
	if (countdown >= MIN_TIMER_VALUE)
	{
                if (!dual_image_mode)
		  dsp_command((DSP_ATTR *)dsp, TIM_ID, SEX, directory, 
                                  filename, imagesave, numbytes, msecs);
		else
                  dsp_command((DSP_ATTR *)dsp, TIM_ID, RRR, directory, 
                                  filename, imagesave, numbytes, msecs);

		notify_set_itimer_func(Expctl->expctl_popup,
			NOTIFY_FUNC_NULL, ITIMER_REAL, NULL, NULL);
	}
	else
	{
 	        if (!dual_image_mode)
                  dsp_command((DSP_ATTR *)dsp, TIM_ID, SEX, directory,
                                  filename, imagesave, numbytes, msecs);
                else
                  dsp_command((DSP_ATTR *)dsp, TIM_ID, RRR, directory,
                                  filename, imagesave, numbytes, msecs);
	}

}

		
#ifdef	__STDC__
int do_save(expctl_opaque *expctl, setup_opaque *setup, fits_opaque *fits)
#else	/* K&R C */
int
do_save(expctl, setup, fits)
expctl_opaque	*expctl;
setup_opaque	*setup;
fits_opaque	*fits;
#endif	/* __STDC__ */
{
	int	ncols, nrows, nccds, max_cols, max_lines,
		istart=1, jstart=1;
	float	msecs;



	/* These variables are for setting the date and time in 
	 * the FITS header.
	 */
	char		gmt[80], date[80];
	struct tm	*gm_time;
	time_t		tp;
	
	/* Get date and time */
	time(&tp);
	gm_time = gmtime(&tp);

	strftime(date, 70, "%d/%m/%y", gm_time);
	strftime(gmt, 70, "%T", gm_time);

	message_write("Saving image header to disk...\n");

	ncols = (int)get_setup_value(setup, NCOLS);
	nrows = (int)get_setup_value(setup, NROWS);
	nccds = 1; /*(int)get_setup_value(setup, NCCDS); */
	msecs = (int)get_expctl_value((EXPCTL_OBJ *)expctl, EXPOSURE_TIME);
	msecs = msecs/1000;
#ifndef	IRTOOL
	/* Not implemented for NICMOS3 Array */
	max_cols = dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID,
		RDM, Y, 1);
	max_lines = dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID,
		RDM, Y, 2);
	istart = dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID, RDM,Y,8);
	jstart = dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID, RDM,Y,7);

	if ((istart > max_cols) || (istart <= 0))
		istart = 1;
	if ((jstart > max_lines) || (jstart <= 0))
		jstart = 1;
#endif	/* IRTOOL */
	
/* NOTE:  PRESCAN and POSTSCAN are currently hardwired for the 2K ccd */
	if (nccds == 4)
	{
		fits_set(fits,
			FITS_DIRECTORY,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_DIRECTORY),
			FITS_FILENAME,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_FILENAME),

			/* FITS Tags */
			FITS_EXPTIME, msecs,
			FITS_IRAFNAME,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_FILENAME),
			FITS_ISTART, istart,
			FITS_JSTART, jstart,
			FITS_NAXIS1, ncols * 2,
			FITS_NAXIS2, nrows * 2,
			FITS_POSTSCAN, 32,
			FITS_PRESCAN, 20,
			FITS_UTSTART, gmt,
			FITS_DATE_OBS, date,
		NULL);
		fits_panel_update();
	}
	else
	{
		fits_set(fits,
			FITS_DIRECTORY,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_DIRECTORY),
			FITS_FILENAME,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_FILENAME),

			/* FITS Tags */
			FITS_EXPTIME, msecs,
			FITS_IRAFNAME,
				(char *)get_expctl_value((EXPCTL_OBJ *)expctl,
				IMAGE_FILENAME),
			FITS_ISTART, istart,
			FITS_JSTART, jstart,
			FITS_NAXIS1, ncols,
			FITS_NAXIS2, nrows * nccds,
			FITS_POSTSCAN, 32,
			FITS_PRESCAN, 20,
			FITS_UTSTART, gmt,
			FITS_DATE_OBS, date,
		NULL);
		fits_panel_update();
	}
	fits_open((FITS_OBJ *)fits);
	if ((fits_write((FITS_OBJ *)fits)) == -1)
	{
		message_write(Expctl->expctl_popup,
			MESSAGE, "Cannot save image to",
			MESSAGE, "specified file.",
		NULL);
		message_write("Save ABORTED.\n");
	}
	else
	{
		if ((int)get_expctl_value((EXPCTL_OBJ *)expctl,
		    DO_AUTO_INCREMENT))
		{
			increment_filename((expctl_opaque)expctl);
		}
		message_write("Save complete.\n");
	}
	fits_close((FITS_OBJ *)fits);
}

/**********************************/
#ifdef  __STDC__
int do_display(fits_opaque *fits)
#else   /* K&R C */
int
do_display(fits)
fits_opaque *fits;
#endif  /* __STDC__ */
{
#ifdef  SBUSTOOL
 
int     k;
int     naxis1,naxis2,bitpix,numbytes;
int     w,h;
int     fb_conf,fb_w,fb_h;
int     min=0,max=~0;
 
/* external variables to hold image buffer from libdsp.c */
 
extern union   exp_data {
        char    ch[MAX_EXPOSURE_BYTES];
        u_int   uin[MAX_EXPOSURE_BYTES/4];
        u_short ush[MAX_EXPOSURE_BYTES/2];
};
extern union   exp_data        expdata;
 
 
        message_write("Display in progress\n");
 
        naxis1 = (fits_opaque) fits_get(fits, FITS_NAXIS1);
        naxis2 = (fits_opaque) fits_get(fits, FITS_NAXIS2);
        bitpix = (fits_opaque) fits_get(fits, FITS_BITPIX);
        numbytes = naxis1 * naxis2 * (bitpix / 8);

        /*
         * if no connection has been established yet.
         */
 
        if (si_datain == -1) {
          message_write("openning connection\n");
          if (si_open () == FAIL) {
            message_write("Open Connection to Ximtool failed\n");
            return(0);
          }
        }
 
        /*
         * if there is a connection, clear the old image.
         */
 
        if (si_datain != -1) {
          if (si_send_erase() == FAIL) {
            message_write("Couldn't clear image\n");
            return(0);
          }
        } 
 
        w = naxis1;
        h = naxis2;
 
        /*
         * send the size of image ...
         */
 
         if (si_set_fb_index(w,h,&fb_conf,&fb_w,&fb_h) == FAIL) {
           message_write("Setting the buffer Size failed\n");
           return(0);
         }

        /* Deinterlacing - 2 readouts (split parallel) */
        /*
        up = 0;
        down = (numbytes/2) - 1;

        for (k=0; k < (numbytes/2); k+=2) {
           im[up++] = expdata.ush[k];
           im[down--] = expdata.ush[k+1];
        }
	*/ 

        message_write("Sending the Image\n");
        if (si_send_image (w,h,(Rawpix_t *) &expdata.ush[0]) == FAIL) {
            message_write("Send Image to Ximtool failed\n");
            return(0);
        }
 
#else   /* VME */
 
        message_write("Display is not implemented for VME boards\n");
 
#endif  /* SBUSTOOL */
} 


#ifdef	__STDC__
int increment_filename(expctl_opaque expctl)
#else	/* K&R C */
int
increment_filename(expctl)
expctl_opaque	expctl;
#endif	/* __STDC__ */
{
	char		*filename, newfilename[255];
	char		buffer[255];
	char		*ptr, *filenameptr;
	int		value;

	filename = (char *)get_expctl_value((EXPCTL_OBJ *)expctl,
		IMAGE_FILENAME);
	if (strlen(filename) == 0)
		return 0;

	strncpy(newfilename, filename, strlen(filename)+1);
	filenameptr = newfilename;

	ptr = filenameptr + strlen(filenameptr) - 1;

	if (!isdigit(*ptr))
		return 0;

	while (isdigit(*ptr) && ptr >= filenameptr) 
		ptr--;
	ptr++;

	while (*ptr == '0')
		ptr++;

	value = atoi(ptr) + 1;
	sprintf(buffer, "%d", value);
	if ((int) strlen(buffer) > (int) strlen(ptr))  {
		if (isdigit(*(ptr-1)))
			ptr--;
	}
	strncpy(ptr, buffer, strlen(buffer));

	set_expctl_panel(expctl, IMAGE_FILENAME, newfilename, NULL);
	return 1;
}

#ifdef  __STDC__
int save_busy(int value)
#else   /* K&R C */
int
save_busy(value)
int     value;
#endif  /* __STDC__ */
{
        ccdtool_busy(value);
	xv_set(Expctl->save_to_disk_button, PANEL_INACTIVE, FALSE, NULL); 
        if (value) {
                xv_set(Expctl->save_to_disk_button, XV_SHOW, FALSE, NULL);
                xv_set(Expctl->abort_button3, XV_SHOW, TRUE, NULL);
        }
        else {
                xv_set(Expctl->abort_button3, XV_SHOW, FALSE, NULL);
                xv_set(Expctl->save_to_disk_button, XV_SHOW, TRUE, NULL);
        }
        dsp_set(ccd_obj->dsp_obj, DSP_ABORT, FALSE, NULL); 
}

#ifdef	__STDC__
int read_busy(int value)
#else	/* K&R C */
int
read_busy(value)
int	value;
#endif	/* __STDC__ */
{
	ccdtool_busy(value);
	xv_set(Expctl->readout_ccd_button, PANEL_INACTIVE, FALSE, NULL);
	if (value) {
		xv_set(Expctl->readout_ccd_button, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->abort_button2, XV_SHOW, TRUE, NULL);
	}
	else {
		xv_set(Expctl->abort_button2, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->readout_ccd_button, XV_SHOW, TRUE, NULL);
	}
	 dsp_set(ccd_obj->dsp_obj, DSP_ABORT, FALSE, NULL); 
}

#ifdef  __STDC__
int readout_busy(int value)
#else	/* K&R C */
int 
readout_busy(value)
int value;
#endif  /* __STDC__ */
{
	if(value){
#       ifdef SBUSTOOL
                xv_set(Expctl->abort_button, PANEL_INACTIVE, TRUE, NULL);
# 	else /* CCDTOOL */
		xv_set(Expctl->abort_button,XV_SHOW,FALSE,NULL);
		xv_set(Expctl->abort_readout,XV_SHOW,TRUE,NULL);
#	endif /* SBUSTOOL */
                xv_set(Expctl->pause_button, PANEL_INACTIVE, TRUE, NULL);
                xv_set(Expctl->change_time_field, PANEL_INACTIVE, TRUE, NULL);
                xv_set(Expctl->change_exp_time, PANEL_INACTIVE, TRUE, NULL);
	}
}


#ifdef	__STDC__
int expose_busy(int value)
#else	/* K&R C */
int
expose_busy(value)
int	value;
#endif	/* __STDC__ */
{
	ccdtool_busy(value);
#ifdef SBUSTOOL
         xv_set(Expctl->abort_button, PANEL_INACTIVE, FALSE, NULL);
#endif
	 xv_set(Expctl->pause_button, PANEL_INACTIVE, FALSE, NULL);
         xv_set(Expctl->change_time_field, PANEL_INACTIVE, FALSE, NULL);
         xv_set(Expctl->change_exp_time, PANEL_INACTIVE, FALSE, NULL);

	if (value) {
		xv_set(Expctl->expose_button, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->abort_button, XV_SHOW, TRUE, NULL);
                xv_set(Expctl->pause_button, XV_SHOW, TRUE, NULL); 
                xv_set(Expctl->exposure_time_setting, XV_SHOW, FALSE, NULL);
                xv_set(Expctl->change_exp_time, XV_SHOW, TRUE, NULL);
                xv_set(Expctl->change_time_field, XV_SHOW, TRUE, NULL);
	}
	else {
		xv_set(Expctl->pause_button, XV_SHOW, FALSE, NULL);
                xv_set(Expctl->change_exp_time, XV_SHOW, FALSE, NULL);
                xv_set(Expctl->change_time_field, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->abort_readout, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->abort_button, XV_SHOW, FALSE, NULL);
		xv_set(Expctl->expose_button, XV_SHOW, TRUE, NULL);
		xv_set(Expctl->exposure_time_setting, XV_SHOW,TRUE, NULL);
	}
	dsp_set(ccd_obj->dsp_obj, DSP_ABORT, FALSE, NULL); 
}

#ifdef  __STDC__
int pause_expose(void)
#else   /* K&R C */
int
pause_expose()
#endif  /* __STDC__ */
{
        Xv_notice       notice;
        int             value;
 
        EXPCTL_OBJ      *expctl_pause;
        dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID, PEX);
 
 
        expctl_pause = NULL;
        expctl_pause = (EXPCTL_OBJ *) Expctl->expctl_popup;
 
        notice = xv_create((Xv_object) expctl_pause, NOTICE,
                        NOTICE_MESSAGE_STRINGS,
                        "Exposure Paused!...",
                        NULL,
                        NOTICE_BUTTON, "RESUME",1,
                        NOTICE_STATUS,&value,
                        XV_SHOW,TRUE, NULL);
 
        if (value)
        {
                dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, TIM_ID,REX);
        }
        notify_set_itimer_func(Expctl->expctl_popup,
                        (Notify_func) countdown,ITIMER_REAL,
                        &timer,NULL);
}


#ifdef	__STDC__
int abort_expose(void)
#else	/* K&R C */
int
abort_expose()
#endif	/* __STDC__ */
{
	Xv_notice	notice;
	int		value;
	
	EXPCTL_OBJ	*expctl_abort;

	expctl_abort = NULL;
	expctl_abort = (EXPCTL_OBJ *) Expctl->expctl_popup;

	notice = xv_create((Xv_object) expctl_abort, NOTICE,
			   NOTICE_MESSAGE_STRINGS,
			   "Really Abort?",
			   NULL,
			   NOTICE_BUTTON, "No", 0,
			   NOTICE_BUTTON, "Yes", 1,
			   NOTICE_STATUS, &value,
			   XV_SHOW, TRUE,
	NULL);


	if (value == NOTICE_YES)
	{
#ifdef  SBUSTOOL
                dsp_command((DSP_ATTR *)ccd_obj->dsp_obj,TIM_ID,ABR);
                dsp_set(ccd_obj->dsp_obj, DSP_ABORT, TRUE, NULL);
                ccd_obj->next_func_ptr = NULL;
 
#else /*VMETOOL*/
                dsp_command((DSP_ATTR *)ccd_obj->dsp_obj, VME_ID, AEX);
                dsp_set(ccd_obj->dsp_obj, DSP_ABORT, 1, NULL);
		ccd_obj->next_func_ptr = NULL;
		
#endif /*SBUSTOOL*/
	}
#ifndef SBUSTOOL
/*        else
        {
         notify_set_itimer_func(Expctl->expctl_popup,
                        (Notify_func) countdown_update, ITIMER_REAL,
                        &timer, NULL);
        }
*/
#endif /*SBUSTOOL*/
}

#ifdef  __STDC__
int abort_read_save(void)
#else   /* K&R C */
int
abort_read_save()
#endif  /* __STDC__ */
{
#       ifdef   SBUSTOOL
        send_reset(ccd_obj->dsp_obj); 
#       endif   /* SBUSTOOL */
        dsp_set(ccd_obj->dsp_obj, DSP_ABORT, TRUE, NULL);
        ccd_obj->next_func_ptr = NULL;
}

#ifdef	__STDC__
void countdown_update(void)
#else	/* K&R C */
void
countdown_update()
#endif	/* __STDC__ */
{
	char	buffer[255];
	int	msecs;


	if ((countdown > 0) &&
	    ((int)dsp_get(ccd_obj->dsp_obj, DSP_FIRST_DONE)))
	{
		check_with_dsp++;
		countdown--;

#if TOM_TIMER
		if (((check_with_dsp % 10) == 0) &&(countdown != 0))
		{
			printf("Checking elapsed time..\n");
			msecs = (target_time -
				dsp_command((DSP_ATTR *)ccd_obj->dsp_obj,
					TIM_ID, RDM, Y, 23)) / 1000;
			printf("Elapsed time read.. msecs=%d, countdown = %d\n",
			       msecs, countdown);
			if (msecs < countdown)
				countdown = msecs;
		}
		printf("\nEnd Tom's Timer\n");
#endif TOM_TIMER
#if TOM_TIMER1
		sprintf(buffer, "Exposure time remaining (secs):  %d %d",
		        countdown, check_with_dsp);
#endif TOM_TIMER1
		sprintf(buffer, "Exposure time remaining (secs):  %d",
		        countdown);
		write_frame(Expctl->expctl_popup, FRAME_LEFT_FOOTER, buffer);
	}
	else
	{
		check_with_dsp = 0;
		notify_set_itimer_func((Notify_client)  Expctl->expctl_popup,
			NOTIFY_FUNC_NULL, ITIMER_REAL, NULL, NULL);
#ifdef	IRTOOL
		sprintf(buffer, "Reading out NICMOS3 Array...");
#else	/* CCDTOOL */
		sprintf(buffer, "Reading out IR Array!....");
		message_write("Reading out IR Array!...\n");
		readout_busy(TRUE);


#endif	/* IRTOOL */
		write_frame(Expctl->expctl_popup, FRAME_LEFT_FOOTER, buffer);
	}
}



/*
 * $Log: expctl.c,v $
 * Revision 1.46  1997/01/03 22:38:37  gumrukcu
 * ***ABDUL***
 * Adding timeout and disply.
 *
 * Revision 1.43  1996/11/25 22:15:51  peker
 * Added new functions .
 *
 * Revision 1.44  1996/06/20  22:12:16  peker
 * Displays the right time on change exposure time field
 *
 * Revision 1.43  1996/06/19 19:52:10  peker
 * fixed some bugs for sbustool
 *
 * Revision 1.42  1996/06/14 23:36:44  peker
 * Recovered from corrupted RCS file.
 * Includes updating FITS date_obs and utstart variables.
 *
 * Revision 1.41  1996/05/13 16:48:39  tjk
 * Added date and time update to fits header.
 *
 * Revision 1.40  1996/05/01 20:12:07  peker
 * Just atest
 *
 * Revision 1.39  1996/03/09  00:42:55  tjk
 * Changed CCD to NICMOS3 for IRTOOL.
 *
 * Revision 1.38  1996/03/08  22:24:54  tjk
 * Removed dsp_commands to read memory Y1, Y2, Y7, and Y8 from the timing
 * board for IRTOOL.
 *
 * Revision 1.37  1996/02/14  23:22:09  tjk
 * Corrected VME/SBUS do_save/do_readout problem.  The SBUS needs to save
 * before readout, but the VME saves after.
 *
 * Revision 1.36  1996/02/14 22:39:39  gumrukcu
 * do_readout_ccd is coming after the do_save.
 *
 * Revision 1.35  1996/02/14 19:42:16  tjk
 * Switched DO_SAVE to come after DO_READOUT.
 *
 * Revision 1.34  1996/02/13  19:28:32  gumrukcu
 * Syntax error for save_to_disk function is corrected.
 *
 * Revision 1.33  1996/02/12 22:34:32  gumrukcu
 * Change in read_out ccd function. do_save is put in the save_to_disk function.
 *
 * Revision 1.31  1996/02/09  22:34:37  gumrukcu
 * When you click on the save_to_disk or read_out_ccd button on the expctl window, all
 * the other buttons are made inactive but the abort read button. The same change is made
 * for the save to disk button.
 *
 * Revision 1.30  1996/02/09  18:58:30  gumrukcu
 * Change in save_to_disk function.
 *
 * Revision 1.28  1996/02/08  20:23:31  gumrukcu
 * The callback functions are changed for save_to_disk and read_out_ccd
 * buttons for VMETOOL.
 *
 * Revision 1.27  1996/02/07  22:23:56  gumrukcu
 * synatax error is corrected.
 *
 * Revision 1.26  1996/02/07  22:05:28  gumrukcu
 * Some addtions to the do_save_disk and read_outccd functions
 * for the VMETOOL.
 *
 * Revision 1.25  1996/02/05  18:40:12  gumrukcu
 * save_busy function is added.
 *
 * Revision 1.24  1996/02/05  00:35:52  gumrukcu
 * Syntax error is corrected in line 818.
 *
 * Revision 1.23  1996/02/05  00:30:27  gumrukcu
 * abort_read_save function is added.
 *
 * Revision 1.22  1996/02/01  22:23:46  gumrukcu
 * Some additions for the new aboort button for readoutccd button.
 *
 * Revision 1.21  1996/01/30  22:57:32  gumrukcu
 * When reading out save header is added to read_out_ccd.
 *
 * Revision 1.20  1996/01/29  23:29:01  gumrukcu
 * Some changes for read_out_ccd function.
 *
 * Revision 1.19  1996/01/18  01:22:17  gumrukcu
 * TImer stuff is commented in read_out_ccd function.
 *
 * Revision 1.18  1996/01/04  20:24:54  gumrukcu
 * expctl in get_expctl_value function is cast to (EXPCTL_OBJ *).
 *
 * Revision 1.17  1996/01/04 19:47:55  gumrukcu
 * casting to DSP_ATTR * is made to dsp in dsp_command.
 *
 * Revision 1.16  1996/01/03 20:21:33  gumrukcu
 * save_to_disk and read_out_ccd are isolated for SBUSTOOL.
 *
 * Revision 1.15  1996/01/03  01:14:25  gumrukcu
 * save_to_disk and shift_ccd functions are added.
 *
 * Revision 1.14  1996/01/02  23:13:37  gumrukcu
 * Some changes made in read_out_ccd function.
 *
 * Revision 1.13  1995/12/29  18:54:25  gumrukcu
 * read_out_ccd function has been modified.
 *
 * Revision 1.12  1995/12/29  01:00:27  gumrukcu
 * The function read_out_ccd is added in. It is the function from the
 * callback function of read_out_ccd button in expctl window.
 *
 * Revision 1.11  1995/12/26  23:09:20  gumrukcu
 *  The functions of the fits window is taken care of for sbustool.
 *
 * Revision 1.10  1995/12/12  22:33:30  tjk
 * Merged SBUS and VME versions
 *
 * Revision 1.9  1995/08/21  18:42:12  tjk
 * converting to ANSI C standard
 *
 * Revision 1.8  1995/08/10  16:53:26  tjk
 * *** empty log message ***
 *
 * Revision 1.7  1995/07/01  02:19:51  tjk
 * Added more information to the temperature out of range message.
 *
 * Revision 1.6  1995/06/30  00:03:52  tjk
 * Fixed the Shutter Control bug which wrote out only 0 or 1 for the shutter
 * only.
 *
 * Revision 1.5  1995/06/12  00:52:04  tjk
 * Typecasted Expctl->expctl_popup as EXPCTL_OBJ
 *
 * Revision 1.4  1995/06/08  16:28:20  tjk
 * Added temperature readout functions...
 *
 * Revision 1.3  1995/05/23  21:49:56  tjk
 * completed temperature upgrade.
 *
 * Revision 1.2  1995/02/22  20:38:17  tjk
 * Added a more complete FITS header.
 *
 * Revision 1.1  1995/02/10  03:56:15  tjk
 * Initial revision
 *
 *
 */
