/* $Header: /rosie/src/ccd_utilities/tool/RCS/fits_panel.c,v 0.3 1996/05/07 17:18:45 tjk Exp $ */

#include <xview/xview.h>
#include <xview/panel.h>

#include "ccd_error.h"
#include "ccd.h"
#include "gfm.h"
#include "ccd_conf.h"
#include "fits.h"

/*>>>>>  Function Prototypes <<<<<*/
#ifdef	__STDC__
void fits_panel_update(void);
void store_fits_settings(FITS_OBJ *fits);
void set_fits_panel(...);
void reset_fits_panel(FITS_OBJ *fits);
void fits_panel_buttons(void);
char simple_value(int value);
int simple_panel(char value);
int bitpix_value(int value);
int bitpix_panel(int value);

#else	/* K&R C */
void fits_panel_update();
void store_fits_settings();
void set_fits_panel();
void reset_fits_panel();
void fits_panel_buttons();
char simple_value();
int simple_panel();
int bitpix_value();
int bitpix_panel();
#endif	/* __STDC__ */
/*>>>>>  End Function Prototypes <<<<<*/

#ifdef __STDC__
void fits_panel_update(void)
#else	/* K&R C */
void fits_panel_update()
#endif	/* __STDC__ */
{
	if (Fits != NULL)
		reset_fits_panel((FITS_OBJ *)ccd_obj->fits_obj);
	else
		fprintf(stderr, "fits_panel.c:  Fits pointer is null.\n");
}

#ifdef	__STDC__
void update_fits_object(void)
#else	/* K&R C */
void update_fits_object()
#endif	/* __STDC__ */
{
	store_fits_settings((FITS_OBJ *)ccd_obj->fits_obj);
}

#ifdef __STDC__
void store_fits_settings(FITS_OBJ *fits)
#else	/* K&R C */
void store_fits_settings(fits)
FITS_OBJ	*fits;
#endif	/* __STDC__ */
{
	double	airmass=0, epoch=0, exposure_time=0;
	char	buffer[255];

#ifdef	DEBUG_FITS_HEADER
	int i;
	char keyword[10], value[72];
#endif	DEBUG_FITS_HEADER

	/*************************************************\
	 * Floats/Doubles need to be handled differently *
	\*************************************************/

	strcpy(buffer, (char *)xv_get(Fits->fits_airmass_text, PANEL_VALUE));
	sscanf(buffer, "%lf", &airmass);
	strcpy(buffer, (char *)xv_get(Fits->fits_epoch_text, PANEL_VALUE));
	sscanf(buffer, "%lf", &epoch);
	strcpy(buffer, (char *)xv_get(Fits->fits_exptime_text, PANEL_VALUE));
	sscanf(buffer, "%lf", &exposure_time);

#ifdef	DEBUG_FITS_HEADER
	printf("\n%lf, %lf, %lf\n", airmass, epoch, exposure_time);
#endif	DEBUG_FITS_HEADER

	fits_set(fits,
		FITS_AIRMASS, airmass,
		FITS_BITPIX, (int)bitpix_value(xv_get(Fits->fits_bitpix_setting,
			PANEL_VALUE)),
		FITS_DATE_OBS, (char *) xv_get(Fits->fits_date_obs_text,
			PANEL_VALUE),
		FITS_DEC, (char *) xv_get(Fits->fits_dec_text,
			PANEL_VALUE),
		FITS_DETECTOR, (char *) xv_get(Fits->fits_detector_text,
			PANEL_VALUE),
		FITS_EPOCH, epoch,
		FITS_EXPTIME, exposure_time,
		FITS_EXPTYPE, (char *) xv_get(Fits->fits_exptype_text,
			PANEL_VALUE),
		FITS_FILTER, (char *) xv_get(Fits->fits_filter_text,
			PANEL_VALUE),
		FITS_INSTRUMENT, (char *) xv_get(Fits->fits_instrument_text,
			PANEL_VALUE),
		FITS_IRAFNAME, (char *) xv_get(Fits->fits_irafname_text,
			PANEL_VALUE),
		FITS_ISTART, (int) char2int((char *)
			xv_get(Fits->fits_istart_text, PANEL_VALUE)),
		FITS_JSTART, (int) char2int((char *)
			xv_get(Fits->fits_jstart_text, PANEL_VALUE)),
		FITS_NAXIS, (int) char2int((char *)
			xv_get(Fits->fits_naxis_text, PANEL_VALUE)),
		FITS_NAXIS1, (int) char2int((char *)
			xv_get(Fits->fits_naxis1_text, PANEL_VALUE)),
		FITS_NAXIS2, (int) char2int((char *)
			xv_get(Fits->fits_naxis2_text, PANEL_VALUE)),
		FITS_OBJECT, (char *) xv_get(Fits->fits_object_text,
			PANEL_VALUE),
		FITS_OBSERVATORY, (char *) xv_get(Fits->fits_observatory_text,
			PANEL_VALUE),
		FITS_OBSERVER, (char *) xv_get(Fits->fits_observer_text,
			PANEL_VALUE),
		FITS_POSTSCAN, (int) char2int((char *)
			xv_get(Fits->fits_postscan_text, PANEL_VALUE)),
		FITS_PRESCAN, (int) char2int((char *)
			xv_get(Fits->fits_prescan_text, PANEL_VALUE)),
		FITS_RA, (char *) xv_get(Fits->fits_ra_text,
			PANEL_VALUE),
		FITS_SIDEREAL_TIME, (char *)
			xv_get(Fits->fits_sidereal_time_text, PANEL_VALUE),
		FITS_SIMPLE, (char) simple_value(
			      xv_get(Fits->fits_simple_setting, PANEL_VALUE)),
		FITS_TELESCOPE, (char *) xv_get(Fits->fits_telescope_text,
			PANEL_VALUE),
		FITS_UTSTART, (char *) xv_get(Fits->fits_ut_start_text,
			PANEL_VALUE),

		FITS_USER_KEYWORD1, (char *) xv_get(Fits->keyword1,
			PANEL_VALUE),
		FITS_USER_KEYWORD2, (char *) xv_get(Fits->keyword2,
			PANEL_VALUE),
		FITS_USER_KEYWORD3, (char *) xv_get(Fits->keyword3,
			PANEL_VALUE),
		FITS_USER_KEYWORD4, (char *) xv_get(Fits->keyword4,
			PANEL_VALUE),
		FITS_USER_KEYWORD5, (char *) xv_get(Fits->keyword5,
			PANEL_VALUE),
		FITS_USER_KEYWORD6, (char *) xv_get(Fits->keyword6,
			PANEL_VALUE),
		FITS_USER_KEYWORD7, (char *) xv_get(Fits->keyword7,
			PANEL_VALUE),
		FITS_USER_KEYWORD8, (char *) xv_get(Fits->keyword8,
			PANEL_VALUE),
		FITS_USER_KEYWORD9, (char *) xv_get(Fits->keyword9,
			PANEL_VALUE),

		FITS_USER_KEYWORD_VALUE1,
			(char *) xv_get(Fits->keyword_value1, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE2,
			(char *) xv_get(Fits->keyword_value2, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE3,
			(char *) xv_get(Fits->keyword_value3, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE4,
			(char *) xv_get(Fits->keyword_value4, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE5,
			(char *) xv_get(Fits->keyword_value5, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE6,
			(char *) xv_get(Fits->keyword_value6, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE7,
			(char *) xv_get(Fits->keyword_value7, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE8,
			(char *) xv_get(Fits->keyword_value8, PANEL_VALUE),
		FITS_USER_KEYWORD_VALUE9,
			(char *) xv_get(Fits->keyword_value9, PANEL_VALUE),
		NULL);
}

#ifdef __STDC__
void set_fits_panel(...)
#else	/* K&R C */
void set_fits_panel(va_alist)
va_dcl
#endif	/* __STDC__ */
{
	va_list	ap;
	int		modifier=0;
	int		value;
	FITS_OBJ	*fits;
	char		buffer[255];

	char		char_value;
	char		*string_value;
	int		int_value;
	double		double_value;


#ifdef	__STDC__
	va_start(ap, NULL);
#else	/* K&R C */
	va_start(ap);
#endif	/* __STDC__ */

	fits = va_arg(ap, FITS_OBJ *);
	modifier = va_arg(ap, int);

	while (modifier != NULL)
	{
		switch (modifier)
		{
			/* Logicals -- True/False */
			/* Integer Values */
			/* Character Strings */
			/* Floating point numbers */
			case FITS_AIRMASS:
				double_value = va_arg(ap, double);
				sprintf(buffer, "%4.2lf", double_value);
				xv_set(Fits->fits_airmass_text, PANEL_VALUE,
				       buffer, NULL);
				break;

			case FITS_BITPIX:
				xv_set(Fits->fits_bitpix_setting, PANEL_VALUE,
				       va_arg(ap, int), NULL);
				break;

			case FITS_DATE_OBS:
				xv_set(Fits->fits_date_obs_text,
				       PANEL_VALUE, va_arg(ap, char *),
				       NULL);
				break;

			case FITS_DEC:
				xv_set(Fits->fits_dec_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_DETECTOR:
				xv_set(Fits->fits_detector_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_EPOCH:
				double_value = va_arg(ap, double);
				sprintf(buffer, "%5.1lf", double_value);
				xv_set(Fits->fits_epoch_text, PANEL_VALUE,
				       buffer, NULL);
				break;

			case FITS_EXPTIME:
				double_value = va_arg(ap, double);
				sprintf(buffer, "%8.3lf", double_value);
				xv_set(Fits->fits_exptime_text, PANEL_VALUE,
				       buffer, NULL);
				break;

			case FITS_EXPTYPE:
				xv_set(Fits->fits_exptype_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_FILTER:
				xv_set(Fits->fits_filter_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_INSTRUMENT:
				xv_set(Fits->fits_instrument_text,
				       PANEL_VALUE, va_arg(ap, char *), NULL);
				break;

			case FITS_IRAFNAME:
				xv_set(Fits->fits_irafname_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_ISTART:
				xv_set(Fits->fits_istart_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_JSTART:
				xv_set(Fits->fits_jstart_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_NAXIS:
				xv_set(Fits->fits_naxis_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_NAXIS1:
				xv_set(Fits->fits_naxis1_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_NAXIS2:
				xv_set(Fits->fits_naxis2_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_OBJECT:
				xv_set(Fits->fits_object_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_OBSERVATORY:
				xv_set(Fits->fits_observatory_text,
				       PANEL_VALUE, va_arg(ap, char *), NULL);
				break;

			case FITS_OBSERVER:
				xv_set(Fits->fits_observer_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_POSTSCAN:
				xv_set(Fits->fits_postscan_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_PRESCAN:
				xv_set(Fits->fits_prescan_text, PANEL_VALUE,
				       int2char(va_arg(ap, int)), NULL);
				break;

			case FITS_RA:
				xv_set(Fits->fits_ra_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_SIDEREAL_TIME:
				xv_set(Fits->fits_sidereal_time_text,
				       PANEL_VALUE, va_arg(ap, char *), NULL);
				break;

			case FITS_SIMPLE:
				xv_set(Fits->fits_simple_setting, PANEL_VALUE,
				       va_arg(ap, int), NULL);
				break;

			case FITS_TELESCOPE:
				xv_set(Fits->fits_telescope_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_UTSTART:
				xv_set(Fits->fits_ut_start_text, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			/* These fields allow the users to customize the 
			 * FITS header.  However, only text strings are
			 * allowed at this time. (4/30/96)
			 */
			case FITS_USER_KEYWORD1:
				xv_set(Fits->keyword1, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD2:
				xv_set(Fits->keyword2, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD3:
				xv_set(Fits->keyword3, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD4:
				xv_set(Fits->keyword4, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD5:
				xv_set(Fits->keyword5, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD6:
				xv_set(Fits->keyword6, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD7:
				xv_set(Fits->keyword7, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD8:
				xv_set(Fits->keyword8, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD9:
				xv_set(Fits->keyword9, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE1:
				xv_set(Fits->keyword_value1, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE2:
				xv_set(Fits->keyword_value2, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE3:
				xv_set(Fits->keyword_value3, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE4:
				xv_set(Fits->keyword_value4, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE5:
				xv_set(Fits->keyword_value5, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE6:
				xv_set(Fits->keyword_value6, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE7:
				xv_set(Fits->keyword_value7, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE8:
				xv_set(Fits->keyword_value8, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;

			case FITS_USER_KEYWORD_VALUE9:
				xv_set(Fits->keyword_value9, PANEL_VALUE,
				       va_arg(ap, char *), NULL);
				break;


			/* Invalid FITS Parameter */
			default:
			    fprintf(stderr,
			    "\nInvalid FITS tag in set_fits_panel():  %d\n",
			    modifier);
			    break;

		}
		modifier = va_arg(ap, int);
	}
}

#ifdef __STDC__
void fits_panel_buttons(void)
#else	/* K&R C */
void fits_panel_buttons()
#endif	/* __STDC__ */
{
	/* Included for future development */
}

#ifdef __STDC__
void reset_fits_panel(FITS_OBJ *fits)
#else	/* K&R C */
void reset_fits_panel(fits)
FITS_OBJ *fits;
#endif	/* __STDC__ */
{
	char	buffer[255];
	int	setting;

/*****************************************************************\
 ************************* W A R N I N G *************************
 *****************************************************************
 **                                                             **
 ** NB:  fits_get() does not adequately handle floats/doubles.  **
 **                                                             **
 *****************************************************************
\*****************************************************************/

	set_fits_panel(fits,
		FITS_AIRMASS, (double) fits->airmass,
		FITS_BITPIX, (int) fits_get(fits, FITS_BITPIX),
		FITS_DATE_OBS, (char *) fits_get(fits, FITS_DATE_OBS),
		FITS_DEC, (char *) fits_get(fits, FITS_DEC),
		FITS_DETECTOR, (char *) fits_get(fits, FITS_DETECTOR),
		FITS_EPOCH, (double) fits->epoch,
		FITS_EXPTIME, (double) fits->exptime,
		FITS_EXPTYPE, (char *) fits_get(fits, FITS_EXPTYPE),
		FITS_FILTER, (char *) fits_get(fits, FITS_FILTER),
		FITS_INSTRUMENT, (char *) fits_get(fits, FITS_INSTRUMENT),
		FITS_IRAFNAME, (char *) fits_get(fits, FITS_IRAFNAME),
		FITS_ISTART, (int) fits_get(fits, FITS_ISTART),
		FITS_JSTART, (int) fits_get(fits, FITS_JSTART),
		FITS_NAXIS, (int) fits_get(fits, FITS_NAXIS),
		FITS_NAXIS1, (int) fits_get(fits, FITS_NAXIS1),
		FITS_NAXIS2, (int) fits_get(fits, FITS_NAXIS2),
		FITS_OBJECT, (char *) fits_get(fits, FITS_OBJECT),
		FITS_OBSERVATORY, (char *) fits_get(fits, FITS_OBSERVATORY),
		FITS_OBSERVER, (char *) fits_get(fits, FITS_OBSERVER),
		FITS_POSTSCAN, (int) fits_get(fits, FITS_POSTSCAN),
		FITS_PRESCAN, (int) fits_get(fits, FITS_PRESCAN),
		FITS_RA, (char *) fits_get(fits, FITS_RA),
		FITS_SIDEREAL_TIME, (char *) fits_get(fits, FITS_SIDEREAL_TIME),
		FITS_SIMPLE, (int) simple_panel(fits_get(fits, FITS_SIMPLE)),
		FITS_TELESCOPE, (char *) fits_get(fits, FITS_TELESCOPE),
		FITS_UTSTART, (char *) fits_get(fits, FITS_UTSTART),

		FITS_USER_KEYWORD1, (char *) fits_get(fits, FITS_USER_KEYWORD1),
		FITS_USER_KEYWORD2, (char *) fits_get(fits, FITS_USER_KEYWORD2),
		FITS_USER_KEYWORD3, (char *) fits_get(fits, FITS_USER_KEYWORD3),
		FITS_USER_KEYWORD4, (char *) fits_get(fits, FITS_USER_KEYWORD4),
		FITS_USER_KEYWORD5, (char *) fits_get(fits, FITS_USER_KEYWORD5),
		FITS_USER_KEYWORD6, (char *) fits_get(fits, FITS_USER_KEYWORD6),
		FITS_USER_KEYWORD7, (char *) fits_get(fits, FITS_USER_KEYWORD7),
		FITS_USER_KEYWORD8, (char *) fits_get(fits, FITS_USER_KEYWORD8),
		FITS_USER_KEYWORD9, (char *) fits_get(fits, FITS_USER_KEYWORD9),

		FITS_USER_KEYWORD_VALUE1,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE1),
		FITS_USER_KEYWORD_VALUE2,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE2),
		FITS_USER_KEYWORD_VALUE3,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE3),
		FITS_USER_KEYWORD_VALUE4,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE4),
		FITS_USER_KEYWORD_VALUE5,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE5),
		FITS_USER_KEYWORD_VALUE6,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE6),
		FITS_USER_KEYWORD_VALUE7,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE7),
		FITS_USER_KEYWORD_VALUE8,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE8),
		FITS_USER_KEYWORD_VALUE9,
			(char *) fits_get(fits, FITS_USER_KEYWORD_VALUE9),
		NULL);
}

#ifdef	__STDC__
char simple_value(int value)
#else	/* K&R C */
char
simple_value(value)
int value;
#endif	/* __STDC__ */
{
	switch(value)
	{
		case TRUE:
			return 'T';
		case FALSE:
			return 'F';
		default:
			fprintf(stderr,
				"\nERROR:  fits_panel.c(simple_value) %d\n",
				value);
	}
}

#ifdef	__STDC__
int simple_panel(char value)
#else	/* K&R C */
int
simple_panel(value)
char value;
#endif	/* __STDC__ */
{
	switch(value)
	{
		case 'T':
			return TRUE;
		case 'F':
			return FALSE;
		default:
			fprintf(stderr,
				"\nERROR:  fits_panel.c(simple_panel) %c\n",
				value);
	}
}

#ifdef	__STDC__
int bitpix_value(int value)
#else	/* K&R C */
int
bitpix_value(value)
int value;
#endif	/* __STDC__ */
{
	switch(value)
	{
		case BITPIX_SETTING_0:
			return BITPIX_VALUE_0;
		case BITPIX_SETTING_1:
			return BITPIX_VALUE_1;
		case BITPIX_SETTING_2:
			return BITPIX_VALUE_2;
		case BITPIX_SETTING_3:
			return BITPIX_VALUE_3;
		case BITPIX_SETTING_4:
			return BITPIX_VALUE_4;
		default:
			fprintf(stderr,
				"\nERROR:  fits_panel.c (bitpix_value) %d\n",
				value);
	}
}

#ifdef	__STDC__
int bitpix_panel(int value)
#else	/* K&R C */
int
bitpix_panel(value)
int value;
#endif	/* __STDC__ */
{
	switch(value)
	{
		case BITPIX_VALUE_0:
			return BITPIX_SETTING_0;
		case BITPIX_VALUE_1:
			return BITPIX_SETTING_1;
		case BITPIX_VALUE_2:
			return BITPIX_SETTING_2;
		case BITPIX_VALUE_3:
			return BITPIX_SETTING_3;
		case BITPIX_VALUE_4:
			return BITPIX_SETTING_4;
		default:
			fprintf(stderr,
				"\nERROR:  fits_panel.c (bitpix_panel) %d\n",
				value);
	}
}


/*
 * $Log: fits_panel.c,v $
 * Revision 0.3  1996/05/07 17:18:45  tjk
 * Added user supplied keywords.
 *
 * Revision 0.2  1995/08/21 18:42:14  tjk
 * converting to ANSI C standard
 *
 * Revision 0.1  1995/08/01  16:33:39  tjk
 * initial revision...
 *
 */
