/* $Header: /rosie/src/ccd_utilities/tool/RCS/setup.c,v 1.18 1997/01/03 22:22:43 gumrukcu Exp $
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <fcntl.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "ccd.h"
#include "ccd_conf.h"
#include "setup_error.h"
#include "message.h"
#include "temperature.h"
#include "dsp.h"
#include "dsp_conf.h"

int	dsp_bad_header_count = 0;
int	ccd_setup = 0;

		/*>>>>>  Function Prototypes  <<<<<*/

int do_setup(void);
int do_reset_timing(dsp_opaque *dsp);
int do_hardware_test(dsp_opaque *dsp);
int execute_long_test(dsp_opaque *dsp, int value, int id);
int do_camera_on(dsp_opaque *dsp);
int do_timing_board(setup_opaque *setup, dsp_opaque *dsp);
int do_dimensions(SETUP_OBJ *setup, dsp_opaque *dsp);
int check_setup(void);
int show_setup_error(int errno);
int setup_busy(int value);
int abort_setup(void);

		/*>>>>>  End of Function Prototypes  <<<<<*/


#ifdef	__STDC__
int do_setup(void)
#else	/* K&R C */
int
do_setup()
#endif	/* __STDC__ */
{
	setup_opaque	*setup;
	dsp_opaque	*dsp;
	int		errno;

	setup = ccd_obj->setup_obj;
	dsp = ccd_obj->dsp_obj;

	store_long_test_settings(ccd_obj->long_test_obj);

	if ((errno = check_setup()) != 1)
	{
		show_setup_error(errno);
	}
	else
	{
		message_write("**************\n");
		message_write("Starting setup...\n");

		setup_busy(TRUE);
		if((int)get_setup_value(setup,DO_RESET_TIMING) &&
                   !(int)dsp_get(dsp, DSP_ABORT))
                {
                        do_reset_timing(dsp);
                }

		if ((int)get_setup_value(setup, DO_HARDWARE_TEST) &&
		    !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_hardware_test(dsp);
		}

		if ((int)get_setup_value(setup, DO_TIMING_BOARD) &&
		    !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_timing_board(setup, dsp);
		}

		if ((int)get_setup_value(setup, DO_CAMERA_ON) &&
		    !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_camera_on(dsp);
		}

		if ((int)get_setup_value(setup, DO_DIMENSIONS) &&
		    !(int)dsp_get(dsp, DSP_ABORT))
		{
			do_dimensions((SETUP_OBJ *)setup, dsp);
		}

#if 0
		if (!ccd_setup)
		{
			ccd_setup = 1;

			xv_set(Ccd->direct_dsp_button, PANEL_INACTIVE, FALSE,
				NULL);
			xv_set(Ccd->expctl_button, PANEL_INACTIVE, FALSE,
				NULL);
			xv_set(Ccd->fits_button, PANEL_INACTIVE, FALSE,
				NULL);
		}
#endif

		set_setup_value(setup, CCD_SETUP_COMPLETE, TRUE, NULL);
		setup_busy(FALSE);

		message_write("Setup complete.\n");

	}

	ccd_obj->next_func_ptr = NULL;
}

#ifdef __STDC__
int do_reset_timing(dsp_opaque *dsp)
#else   /* K&R C */
int
do_reset_timing(dsp)
dsp_opaque      *dsp;
#endif  /* __STDC__ */
{
        int ww;
        int i;
        message_write("Resetting Timing Board\n");
        ww = 0x7;
        send_ww(dsp,ww);
        send_tdl(dsp,TIM_ID,0x0f0f0f0f);
        ww = 0x0;
        send_ww(dsp,ww);
        if (dsp_get_reply(dsp,TIM_ID,SYR,DEFAULT_TIMEOUT) ==
              DSP_ERROR)
          return DSP_ERROR;
}
 

#ifdef	__STDC__
int do_hardware_test(dsp_opaque *dsp)
#else	/* K&R C */
int
do_hardware_test(dsp)
dsp_opaque	*dsp;
#endif	/* __STDC__ */
{
	int	i, value, setting, errorno;
        setup_opaque       *setup;
	long_test_opaque   *long_test;

	setup = ccd_obj->setup_obj;
	long_test = ccd_obj->long_test_obj;

	message_write("Doing hardware test...\n");

        setting = (int)get_setup_value(setup, HARDWARE_TEST_SETTING);

        if (setting == 0)
	{
           errorno = 0;

	   message_write("     Testing Timing Board...\n");
	   for (i=1; i<=10; i++)
	   {
	      value = dsp_command((DSP_ATTR *)dsp, TIM_ID, TDL, i);
              if (value != i && value != DSP_ABORTED) 
	        errorno++;
	   }
           /*if (errorno != 0)*/
	     message_write("Number of errors for Timing board:  %d of 10\n",
	   				              		errorno);

	   message_write("Hardware test complete.\n");
        }
        else {

	   if ((int)get_long_test_value(long_test, TIMING_LONG_SELECT)) {
	     message_write("     Testing Timing board...\n");
             value = (int)get_long_test_value(long_test, TIMING_LONG_NUM);
             message_write("Number of errors for Timing board:  %d of %d\n", 
				execute_long_test(dsp, value, TIM_ID), value);
           }

	   message_write("Hardware test complete.\n");
	}
}

#ifdef	__STDC__
int execute_long_test(dsp_opaque *dsp, int value, int id)
#else	/* K&R C */
int
execute_long_test(dsp, value, id)
dsp_opaque	*dsp;
int	value, id;
#endif	/* __STDC__ */
{
extern	int	dsp_errno;
int	errorno, number, i, sendnum, addnum;

                   errorno = 0;
		   sendnum = 0;
		   dsp_bad_header_count = 1;

		   /* generate number to test all 24 bits */

                   addnum = (16777000/value);
           	   for(i=1; i<=value; i=i++)
		   {
                      sendnum = (sendnum + addnum);
	              number = dsp_command((DSP_ATTR *)dsp, id, TDL, sendnum);
		      if (dsp_errno == DSP_BAD_REPLY_HEADER)
		      {
				dsp_bad_header_count++;
				dsp_errno = DSP_OK;
		      }
                      if (number != sendnum)
			errorno++;
	           }
		   dsp_bad_header_count = 0;
                   return(errorno);
}

#ifdef	__STDC__
int do_camera_on(dsp_opaque *dsp)
#else	/* K&R C */
int
do_camera_on(dsp)
dsp_opaque	*dsp;
#endif	/* __STDC__ */
{
	int ret_value;
	message_write("Doing camera on...\n");
	ret_value = dsp_command((DSP_ATTR *)(DSP_ATTR *)dsp, TIM_ID, CON);
	if(ret_value != DSP_OK)
		message_write("Camera on can not be completed.\n");
	else
		message_write("Camera on complete.\n");
}


#ifdef	__STDC__
int do_timing_board(setup_opaque *setup, dsp_opaque *dsp)
#else	/* K&R C */
int
do_timing_board(setup, dsp)
setup_opaque	*setup;
dsp_opaque	*dsp;
#endif	/* __STDC__ */
{
	int	value, bit_value,ret_value,flag = 0;
	int	setting, lda_value;
	char	*filename;

	message_write("Configuring Timing board...\n");

	setting = (int)get_setup_value(setup, TIMING_SETTING);
	lda_value = (int)get_setup_value(setup, TIMING_LDA);
	filename = (char *)get_setup_value(setup, TIMING_FILENAME);

	if (setting == CCD_LDA_SETTING)
	{
		ret_value = dsp_command((DSP_ATTR *)dsp, TIM_ID, LDA, lda_value);
		if(ret_value == DSP_ERROR)
			flag = 1;
	}
	else if (setting == CCD_FILENAME_SETTING)
	{
		value = dsp_command((DSP_ATTR *)dsp, TIM_ID, RDM, X, 0);
		bit_value = value & 0x01;
		if (bit_value)
		{
			message_write("     sending STP...\n");
			ret_value = dsp_command((DSP_ATTR *)dsp, TIM_ID, STP);
			if(ret_value != DSP_OK)
				flag = 1;
			message_write("     downloading...\n");
			ret_value =  dsp_download(dsp, TIM_ID, filename);
			if(ret_value == DSP_ERROR)
				flag = 1;
			message_write("     sending IDL...\n");
			ret_value = dsp_command((DSP_ATTR *)dsp, TIM_ID, IDL);
			if(ret_value == DSP_ERROR)
				flag = 1;
		}
		else
		{
			message_write("     downloading...\n");
			ret_value = dsp_download(dsp, TIM_ID, filename);
			if(ret_value == DSP_ERROR)
				flag = 1;
		}
	}
	if(flag == 1)
		message_write("Timing Board can not be configured.\n");
	else
		message_write("Timing Board configured.\n");
}


#ifdef	__STDC__
int do_dimensions(SETUP_OBJ *setup, dsp_opaque *dsp)
#else	/* K&R C */
int
do_dimensions(setup, dsp)
SETUP_OBJ	*setup;
dsp_opaque	*dsp;
#endif	/* __STDC__ */
{
	int	total,ret_value,flag;

	message_write("Setting dimensions...\n");

	dsp_command((DSP_ATTR *)dsp, TIM_ID, WRM, Y, 1, setup->ncols);
	dsp_command((DSP_ATTR *)dsp, TIM_ID, WRM, Y, 2, setup->nrows);

	total = /* setup->nccds */ setup->nrows * setup->ncols;

	if(flag == 1)
		message_write("Dimensions can not be set.\n");
	else
		message_write("Dimensions set.\n");
}


#ifdef	__STDC__
int check_setup(void)
#else	/* K&R C */
int
check_setup()
#endif	/* __STDC__ */
{
        char    	*char_ptr;
        int     	setting, value;
	setup_opaque	*setup;
	char		*filename;
	int		fd;

	setup = ccd_obj->setup_obj;

	if ((int)get_setup_value(setup, DO_TIMING_BOARD))
	{
		setting = (int)get_setup_value(setup, TIMING_SETTING);
		if (setting == CCD_LDA_SETTING)
		{
        		if ((int)get_setup_value(setup,TIMING_LDA) == UNDEFINED)
				return TIMING_LDA_UNDEFINED;
		}
		else if (setting == CCD_FILENAME_SETTING)
		{
			filename = (char *)get_setup_value(setup,
					TIMING_FILENAME);
			if (strlen(filename) == 0)
				return TIMING_FILENAME_UNDEFINED;
			else if ((fd = open(filename, O_RDONLY)) == -1)
				return TIMING_FILENAME_NO_EXIST;
			else close(fd);
		}
	}

	if ((int)get_setup_value(setup, DO_DIMENSIONS))
	{
		if ((int)get_setup_value(setup, NCOLS) == UNDEFINED)
				return NCOLS_UNDEFINED;
		if ((int)get_setup_value(setup, NROWS) == UNDEFINED)
				return NROWS_UNDEFINED;
	}

	return 1;
}


#ifdef	__STDC__
int show_setup_error(int errno)
#else	/* K&R C */
show_setup_error(errno)
int	errno;
#endif	/* __STDC__ */
{
	switch (errno)
	{
		case TIMING_LDA_UNDEFINED:
                	message_popup(Setup->setup_popup,
                        	MESSAGE, "You must specify an application",
				MESSAGE, "number for the Timing board.",
			NULL);
			break;
		case TIMING_FILENAME_UNDEFINED:
                	message_popup(Setup->setup_popup,
                        	MESSAGE, "You must specify an application",
				MESSAGE, "number for the Timing board.",
			NULL);
			break;
		case TIMING_FILENAME_NO_EXIST:
			message_popup(Setup->setup_popup,
				MESSAGE, "The filename specified for the",
				MESSAGE, "Timing board does not exist.",
			NULL);
			break;
		case NCOLS_UNDEFINED:
			message_popup(Setup->setup_popup,
				MESSAGE, "You must specify the number of",
				MESSAGE, "columns for the IR Array image.",
			NULL);
			break;
		case NROWS_UNDEFINED:
			message_popup(Setup->setup_popup,
				MESSAGE, "You must specify the number of",
				MESSAGE, "rows for the IR Array image.",
			NULL);
			break;
		default:
			break;
	}
}


#ifdef	__STDC__
int setup_busy(int value)
#else	/* K&R C */
int
setup_busy(value)
int	value;
#endif	/* __STDC__ */
{
	ccdtool_busy(value);

	if (value)
	{
		xv_set(Setup->apply_button, XV_SHOW, FALSE, NULL);
		xv_set(Setup->select_all_button, XV_SHOW, FALSE, NULL);
		xv_set(Setup->clear_all_button, XV_SHOW, FALSE, NULL);
		xv_set(Setup->reset_button, XV_SHOW, FALSE, NULL);
		xv_set(Setup->abort_button, XV_SHOW, TRUE, NULL);
	}
	else
	{
		xv_set(Setup->abort_button, XV_SHOW, FALSE, NULL);
		xv_set(Setup->apply_button, XV_SHOW, TRUE, NULL);
		xv_set(Setup->select_all_button, XV_SHOW, TRUE, NULL);
		xv_set(Setup->clear_all_button, XV_SHOW, TRUE, NULL);
		xv_set(Setup->reset_button, XV_SHOW, TRUE, NULL);
		dsp_set(ccd_obj->dsp_obj, DSP_ABORT, FALSE, NULL);
	}
}


#ifdef	__STDC__
int abort_setup(void)
#else	/* K&R C */
int
abort_setup()
#endif	/* __STDC__ */
{
	dsp_set(ccd_obj->dsp_obj, DSP_ABORT, TRUE, NULL);
	ccd_obj->next_func_ptr = NULL;
}
