#ifndef TEMPERATURE_C
#define TEMPERATURE_C
/* $Header: /rosie/src/ccd_utilities/tool/RCS/temperature.c,v 1.5 1996/11/05 20:17:07 peker Exp $ */

/*************************************************************************\
 *  This function calculate the temperature of the the ccd using the     *
 *  Chebychev series and the coefficients given for the CY7 Series       *
 *  Silicon Diodes Standard Curve #10 manufactured by Omega Engineering. *
 *  For this diode, there are eleven (11) coefficients.  Also, the adu-  *
 *  to-voltage conversion factor is needed to use the formula given by   *
 *  Omega Engineering.                                                   *
 *                                                                       *
 *  The function ccd_temperature takes an array of these coefficients,   *
 *  the number of these coefficients, upper voltage limit, lower voltage *
 *  limit, and the adu count as parameters.  The function is declared in *
 *  both ANSI C and K&R C for compatability purposes.  __STDC__ is the   *
 *  variable which signifies ANSI C.                                     *
\*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "temperature.h"

#define TOLERANCE	0.5
#define MAX_TOLERANCE_TRIALS	30

#ifdef __STDC__
float ccd_temperature(float temp_coeff[], int n,
		       float vu, float vl,
		       float adu_per_volt,
		       int adu_offset,
		       float adu)
#else
float ccd_temperature(temp_coeff, n, vu, vl, adu_per_volt, adu_offset, adu)
float temp_coeff[], vu, vl, adu_per_volt, adu;
int adu_offset, n;
#endif __STDC__
{
  int i;

  float temperature = -273.15,
	 voltage = 0.0,
	 tc[N_MAX],
	 x;
#ifdef DEBUG_TEMPERATURE
  for (i = 0; i < n; i++)
  {
  printf("temp_coeff[%d] = %e\n", i, temp_coeff[i]);
  printf("v_upper = %e\n", vu);
  printf("v_lower = %e\n", vl);
  printf("adu = %e\t adu_per_volt = %e\n", adu, adu_per_volt);
  }
#endif DEBUG_TEMPERATURE

  /* Convert adu's to voltage */
  voltage = (adu - (float) adu_offset) / adu_per_volt;

  /* Calculate dimensionless variable for the Chebychev series */
  x = ((voltage - vl) - (vu - voltage)) / (vu - vl);

  tc[0] = 1;
  tc[1] = x;

  if (n <=2 )
	{
	temperature = temp_coeff[0] + (temp_coeff[1] * voltage); 
	}
  else
	{
  	for (i = 2; i < n; i++)
  		{
    		tc[i] = 2.0 * x * tc[i-1] - tc[i-2];
    		temperature += temp_coeff[i] * tc[i];
  		}
	}

  return temperature;
}

/*---------------------------------------------------------------------*/

#ifdef __STDC__
int calc_temp_adu(float temp_coeff[], int n,
		       float vu, float vl,
		       float adu_per_volt,
		       int adu_offset,
		       float temperature)
#else
int calc_temp_adu(temp_coeff, n, vu, vl, adu_per_volt, adu_offset, temperature)
float temp_coeff[], vu, vl, adu_per_volt, temperature;
int adu_offset, n;
#endif __STDC__
{
  int tolerance = 0, trials = 0;

  float adu = 0.0,
	 tc[N_MAX],
	 vmid, lower, upper,
	 voltage=0.0,
	 target_temp=-273.15,
	 x;

  lower = vl;
  upper = vu;

  vmid = (lower + upper) * 0.5;

  /* Numerically determine v for a given temperature   */
  /* Choose an initial adu in the middle of vu and vl  */
  adu = vmid * adu_per_volt + (float) adu_offset;

  tolerance = (fabs(target_temp - temperature) > TOLERANCE);

  while (tolerance && (trials < MAX_TOLERANCE_TRIALS))
  {
    target_temp = ccd_temperature(temp_coeff, n, vu, vl,
				  adu_per_volt, adu_offset, adu);

    tolerance = (fabs(target_temp - temperature) > TOLERANCE);

    if (tolerance)
    {
      if (target_temp < temperature)
      { 
        upper = vmid;
        vmid = (lower + upper) * 0.5;
        adu = vmid * adu_per_volt + (float) adu_offset;
      }
      else if (target_temp > temperature)
      {
        lower = vmid;
        vmid = (lower + upper) * 0.5;
        adu = vmid * adu_per_volt + (float) adu_offset;
      }
    }

    trials++;

#ifdef DEBUG_TEMPERATURE
    printf("%d %5s\t%12.3e\t%12.3e%12.3e\n", trials, " ", target_temp, adu,
           temperature);
#endif DEBUG_TEMPERATURE

  }

  return (int) adu;
}

/*---------------------------------------------------------------------*/

#undef TOLERANCE
#undef MAX_TOLERANCE_TRIALS

/*
 * $Log: temperature.c,v $
 * Revision 1.5  1996/11/05 20:17:07  peker
 * included 2 point linear fit if number of coeff <=2 Dahm
 *
 * Revision 1.4  1995/08/21  21:13:42  tjk
 * Corrected log message for new RCS version.
 *
 * Revision 1.3  1995/07/01  02:20:13  tjk
 * Added a maximum number of trials to make when trying to determine the
 * adu for a given temperature.
 *
 * Revision 1.2  1995/05/23  21:52:13  tjk
 * completed temperature upgrade.
 *
 */
#endif TEMPERATURE_C
